// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

use crate::{RenderNode, Shadow};
use glib::translate::*;
use std::fmt;

glib::wrapper! {
    #[doc(alias = "GskShadowNode")]
    pub struct ShadowNode(Shared<ffi::GskShadowNode>);

    match fn {
        ref => |ptr| ffi::gsk_render_node_ref(ptr as *mut ffi::GskRenderNode),
        unref => |ptr| ffi::gsk_render_node_unref(ptr as *mut ffi::GskRenderNode),
    }
}

impl glib::StaticType for ShadowNode {
    fn static_type() -> glib::Type {
        unsafe { from_glib(ffi::gsk_shadow_node_get_type()) }
    }
}

impl ShadowNode {
    #[doc(alias = "gsk_shadow_node_new")]
    pub fn new(child: impl AsRef<RenderNode>, shadows: &[Shadow]) -> ShadowNode {
        skip_assert_initialized!();
        let n_shadows = shadows.len() as _;
        unsafe {
            from_glib_full(ffi::gsk_shadow_node_new(
                child.as_ref().to_glib_none().0,
                shadows.to_glib_none().0,
                n_shadows,
            ))
        }
    }

    #[doc(alias = "gsk_shadow_node_get_child")]
    #[doc(alias = "get_child")]
    pub fn child(&self) -> RenderNode {
        unsafe { from_glib_none(ffi::gsk_shadow_node_get_child(self.to_glib_none().0)) }
    }

    #[doc(alias = "gsk_shadow_node_get_n_shadows")]
    #[doc(alias = "get_n_shadows")]
    pub fn n_shadows(&self) -> usize {
        unsafe { ffi::gsk_shadow_node_get_n_shadows(self.to_glib_none().0) }
    }
}

impl fmt::Display for ShadowNode {
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        f.write_str("ShadowNode")
    }
}
