/* pages */

@keyframes up-and-down {
  from {
    background-position: center 30%, center 30%, center;
  }

  to {
    background-position: center 40%, center 30%, center;
  }
}

@keyframes left-and-right {
  from {
    background-position: 40% 30%, center 30%, center;
  }

  to {
    background-position: 60% 30%, center 30%, center;
  }
}

/* .page:nth-child(2) { background: linear-gradient(to right, #3584e4, #1a5fb4); }  overview */
.page:nth-child(3) {}

/* customize */
/* .page:nth-child(4) { background: linear-gradient(to right, #e66100, #c64600); }  workspaces */

.page:nth-child(5) {
  /* up down */
  background: url('/org/gnome/Tour/hand-fg.svg'),
    url('/org/gnome/Tour/updown-bg.svg');
  background-repeat: no-repeat;
  background-position: center 30%;
  animation: up-and-down 2s ease-in-out infinite alternate;
}

.page:nth-child(6) {
  /* left right */
  background: url('/org/gnome/Tour/hand-fg.svg'),
    url('/org/gnome/Tour/leftright-bg.svg');
  background-repeat: no-repeat;
  background-position: center 30%;
  animation: left-and-right 2s ease-in-out infinite alternate;
}
