// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

#include "manual.h"
#include <stdio.h>

#define PRINT_CONSTANT(CONSTANT_NAME) \
    printf("%s;", #CONSTANT_NAME); \
    printf(_Generic((CONSTANT_NAME), \
                    char *: "%s", \
                    const char *: "%s", \
                    char: "%c", \
                    signed char: "%hhd", \
                    unsigned char: "%hhu", \
                    short int: "%hd", \
                    unsigned short int: "%hu", \
                    int: "%d", \
                    unsigned int: "%u", \
                    long: "%ld", \
                    unsigned long: "%lu", \
                    long long: "%lld", \
                    unsigned long long: "%llu", \
                    float: "%f", \
                    double: "%f", \
                    long double: "%ld"), \
           CONSTANT_NAME); \
    printf("\n");

int main() {
    PRINT_CONSTANT(GLIB_SYSDEF_AF_INET);
    PRINT_CONSTANT(GLIB_SYSDEF_AF_INET6);
    PRINT_CONSTANT(GLIB_SYSDEF_AF_UNIX);
    PRINT_CONSTANT(GLIB_SYSDEF_MSG_DONTROUTE);
    PRINT_CONSTANT(GLIB_SYSDEF_MSG_OOB);
    PRINT_CONSTANT(GLIB_SYSDEF_MSG_PEEK);
    PRINT_CONSTANT((guint) G_ASCII_ALNUM);
    PRINT_CONSTANT((guint) G_ASCII_ALPHA);
    PRINT_CONSTANT((guint) G_ASCII_CNTRL);
    PRINT_CONSTANT((guint) G_ASCII_DIGIT);
    PRINT_CONSTANT(G_ASCII_DTOSTR_BUF_SIZE);
    PRINT_CONSTANT((guint) G_ASCII_GRAPH);
    PRINT_CONSTANT((guint) G_ASCII_LOWER);
    PRINT_CONSTANT((guint) G_ASCII_PRINT);
    PRINT_CONSTANT((guint) G_ASCII_PUNCT);
    PRINT_CONSTANT((guint) G_ASCII_SPACE);
    PRINT_CONSTANT((guint) G_ASCII_UPPER);
    PRINT_CONSTANT((guint) G_ASCII_XDIGIT);
    PRINT_CONSTANT(G_BIG_ENDIAN);
    PRINT_CONSTANT((gint) G_BOOKMARK_FILE_ERROR_APP_NOT_REGISTERED);
    PRINT_CONSTANT((gint) G_BOOKMARK_FILE_ERROR_FILE_NOT_FOUND);
    PRINT_CONSTANT((gint) G_BOOKMARK_FILE_ERROR_INVALID_URI);
    PRINT_CONSTANT((gint) G_BOOKMARK_FILE_ERROR_INVALID_VALUE);
    PRINT_CONSTANT((gint) G_BOOKMARK_FILE_ERROR_READ);
    PRINT_CONSTANT((gint) G_BOOKMARK_FILE_ERROR_UNKNOWN_ENCODING);
    PRINT_CONSTANT((gint) G_BOOKMARK_FILE_ERROR_URI_NOT_FOUND);
    PRINT_CONSTANT((gint) G_BOOKMARK_FILE_ERROR_WRITE);
    PRINT_CONSTANT((gint) G_CHECKSUM_MD5);
    PRINT_CONSTANT((gint) G_CHECKSUM_SHA1);
    PRINT_CONSTANT((gint) G_CHECKSUM_SHA256);
    PRINT_CONSTANT((gint) G_CHECKSUM_SHA384);
    PRINT_CONSTANT((gint) G_CHECKSUM_SHA512);
    PRINT_CONSTANT((gint) G_CONVERT_ERROR_BAD_URI);
    PRINT_CONSTANT((gint) G_CONVERT_ERROR_EMBEDDED_NUL);
    PRINT_CONSTANT((gint) G_CONVERT_ERROR_FAILED);
    PRINT_CONSTANT((gint) G_CONVERT_ERROR_ILLEGAL_SEQUENCE);
    PRINT_CONSTANT((gint) G_CONVERT_ERROR_NOT_ABSOLUTE_PATH);
    PRINT_CONSTANT((gint) G_CONVERT_ERROR_NO_CONVERSION);
    PRINT_CONSTANT((gint) G_CONVERT_ERROR_NO_MEMORY);
    PRINT_CONSTANT((gint) G_CONVERT_ERROR_PARTIAL_INPUT);
    PRINT_CONSTANT(G_CSET_A_2_Z);
    PRINT_CONSTANT(G_CSET_DIGITS);
    PRINT_CONSTANT(G_CSET_a_2_z);
    PRINT_CONSTANT(G_DATALIST_FLAGS_MASK);
    PRINT_CONSTANT((gint) G_DATE_APRIL);
    PRINT_CONSTANT((gint) G_DATE_AUGUST);
    PRINT_CONSTANT(G_DATE_BAD_DAY);
    PRINT_CONSTANT(G_DATE_BAD_JULIAN);
    PRINT_CONSTANT((gint) G_DATE_BAD_MONTH);
    PRINT_CONSTANT((gint) G_DATE_BAD_WEEKDAY);
    PRINT_CONSTANT(G_DATE_BAD_YEAR);
    PRINT_CONSTANT((gint) G_DATE_DAY);
    PRINT_CONSTANT((gint) G_DATE_DECEMBER);
    PRINT_CONSTANT((gint) G_DATE_FEBRUARY);
    PRINT_CONSTANT((gint) G_DATE_FRIDAY);
    PRINT_CONSTANT((gint) G_DATE_JANUARY);
    PRINT_CONSTANT((gint) G_DATE_JULY);
    PRINT_CONSTANT((gint) G_DATE_JUNE);
    PRINT_CONSTANT((gint) G_DATE_MARCH);
    PRINT_CONSTANT((gint) G_DATE_MAY);
    PRINT_CONSTANT((gint) G_DATE_MONDAY);
    PRINT_CONSTANT((gint) G_DATE_MONTH);
    PRINT_CONSTANT((gint) G_DATE_NOVEMBER);
    PRINT_CONSTANT((gint) G_DATE_OCTOBER);
    PRINT_CONSTANT((gint) G_DATE_SATURDAY);
    PRINT_CONSTANT((gint) G_DATE_SEPTEMBER);
    PRINT_CONSTANT((gint) G_DATE_SUNDAY);
    PRINT_CONSTANT((gint) G_DATE_THURSDAY);
    PRINT_CONSTANT((gint) G_DATE_TUESDAY);
    PRINT_CONSTANT((gint) G_DATE_WEDNESDAY);
    PRINT_CONSTANT((gint) G_DATE_YEAR);
    PRINT_CONSTANT(G_E);
    PRINT_CONSTANT((gint) G_ERR_DIGIT_RADIX);
    PRINT_CONSTANT((gint) G_ERR_FLOAT_MALFORMED);
    PRINT_CONSTANT((gint) G_ERR_FLOAT_RADIX);
    PRINT_CONSTANT((gint) G_ERR_NON_DIGIT_IN_CONST);
    PRINT_CONSTANT((gint) G_ERR_UNEXP_EOF);
    PRINT_CONSTANT((gint) G_ERR_UNEXP_EOF_IN_COMMENT);
    PRINT_CONSTANT((gint) G_ERR_UNEXP_EOF_IN_STRING);
    PRINT_CONSTANT((gint) G_ERR_UNKNOWN);
    PRINT_CONSTANT((gint) G_FILE_ERROR_ACCES);
    PRINT_CONSTANT((gint) G_FILE_ERROR_AGAIN);
    PRINT_CONSTANT((gint) G_FILE_ERROR_BADF);
    PRINT_CONSTANT((gint) G_FILE_ERROR_EXIST);
    PRINT_CONSTANT((gint) G_FILE_ERROR_FAILED);
    PRINT_CONSTANT((gint) G_FILE_ERROR_FAULT);
    PRINT_CONSTANT((gint) G_FILE_ERROR_INTR);
    PRINT_CONSTANT((gint) G_FILE_ERROR_INVAL);
    PRINT_CONSTANT((gint) G_FILE_ERROR_IO);
    PRINT_CONSTANT((gint) G_FILE_ERROR_ISDIR);
    PRINT_CONSTANT((gint) G_FILE_ERROR_LOOP);
    PRINT_CONSTANT((gint) G_FILE_ERROR_MFILE);
    PRINT_CONSTANT((gint) G_FILE_ERROR_NAMETOOLONG);
    PRINT_CONSTANT((gint) G_FILE_ERROR_NFILE);
    PRINT_CONSTANT((gint) G_FILE_ERROR_NODEV);
    PRINT_CONSTANT((gint) G_FILE_ERROR_NOENT);
    PRINT_CONSTANT((gint) G_FILE_ERROR_NOMEM);
    PRINT_CONSTANT((gint) G_FILE_ERROR_NOSPC);
    PRINT_CONSTANT((gint) G_FILE_ERROR_NOSYS);
    PRINT_CONSTANT((gint) G_FILE_ERROR_NOTDIR);
    PRINT_CONSTANT((gint) G_FILE_ERROR_NXIO);
    PRINT_CONSTANT((gint) G_FILE_ERROR_PERM);
    PRINT_CONSTANT((gint) G_FILE_ERROR_PIPE);
    PRINT_CONSTANT((gint) G_FILE_ERROR_ROFS);
    PRINT_CONSTANT((gint) G_FILE_ERROR_TXTBSY);
    PRINT_CONSTANT((guint) G_FILE_SET_CONTENTS_CONSISTENT);
    PRINT_CONSTANT((guint) G_FILE_SET_CONTENTS_DURABLE);
    PRINT_CONSTANT((guint) G_FILE_SET_CONTENTS_NONE);
    PRINT_CONSTANT((guint) G_FILE_SET_CONTENTS_ONLY_EXISTING);
    PRINT_CONSTANT((guint) G_FILE_TEST_EXISTS);
    PRINT_CONSTANT((guint) G_FILE_TEST_IS_DIR);
    PRINT_CONSTANT((guint) G_FILE_TEST_IS_EXECUTABLE);
    PRINT_CONSTANT((guint) G_FILE_TEST_IS_REGULAR);
    PRINT_CONSTANT((guint) G_FILE_TEST_IS_SYMLINK);
    PRINT_CONSTANT((guint) G_FORMAT_SIZE_BITS);
    PRINT_CONSTANT((guint) G_FORMAT_SIZE_DEFAULT);
    PRINT_CONSTANT((guint) G_FORMAT_SIZE_IEC_UNITS);
    PRINT_CONSTANT((guint) G_FORMAT_SIZE_LONG_FORMAT);
    PRINT_CONSTANT((guint) G_FORMAT_SIZE_ONLY_UNIT);
    PRINT_CONSTANT((guint) G_FORMAT_SIZE_ONLY_VALUE);
    PRINT_CONSTANT((guint) G_HOOK_FLAG_ACTIVE);
    PRINT_CONSTANT((guint) G_HOOK_FLAG_IN_CALL);
    PRINT_CONSTANT((guint) G_HOOK_FLAG_MASK);
    PRINT_CONSTANT(G_HOOK_FLAG_USER_SHIFT);
    PRINT_CONSTANT(G_IEEE754_DOUBLE_BIAS);
    PRINT_CONSTANT(G_IEEE754_FLOAT_BIAS);
    PRINT_CONSTANT((gint) G_IN_ORDER);
    PRINT_CONSTANT((gint) G_IO_CHANNEL_ERROR_FAILED);
    PRINT_CONSTANT((gint) G_IO_CHANNEL_ERROR_FBIG);
    PRINT_CONSTANT((gint) G_IO_CHANNEL_ERROR_INVAL);
    PRINT_CONSTANT((gint) G_IO_CHANNEL_ERROR_IO);
    PRINT_CONSTANT((gint) G_IO_CHANNEL_ERROR_ISDIR);
    PRINT_CONSTANT((gint) G_IO_CHANNEL_ERROR_NOSPC);
    PRINT_CONSTANT((gint) G_IO_CHANNEL_ERROR_NXIO);
    PRINT_CONSTANT((gint) G_IO_CHANNEL_ERROR_OVERFLOW);
    PRINT_CONSTANT((gint) G_IO_CHANNEL_ERROR_PIPE);
    PRINT_CONSTANT((guint) G_IO_ERR);
    PRINT_CONSTANT((gint) G_IO_ERROR_AGAIN);
    PRINT_CONSTANT((gint) G_IO_ERROR_INVAL);
    PRINT_CONSTANT((gint) G_IO_ERROR_NONE);
    PRINT_CONSTANT((gint) G_IO_ERROR_UNKNOWN);
    PRINT_CONSTANT((guint) G_IO_FLAG_APPEND);
    PRINT_CONSTANT((guint) G_IO_FLAG_GET_MASK);
    PRINT_CONSTANT((guint) G_IO_FLAG_IS_READABLE);
    PRINT_CONSTANT((guint) G_IO_FLAG_IS_SEEKABLE);
    PRINT_CONSTANT((guint) G_IO_FLAG_IS_WRITABLE);
    PRINT_CONSTANT((guint) G_IO_FLAG_IS_WRITEABLE);
    PRINT_CONSTANT((guint) G_IO_FLAG_MASK);
    PRINT_CONSTANT((guint) G_IO_FLAG_NONBLOCK);
    PRINT_CONSTANT((guint) G_IO_FLAG_NONE);
    PRINT_CONSTANT((guint) G_IO_FLAG_SET_MASK);
    PRINT_CONSTANT((guint) G_IO_HUP);
    PRINT_CONSTANT((guint) G_IO_IN);
    PRINT_CONSTANT((guint) G_IO_NVAL);
    PRINT_CONSTANT((guint) G_IO_OUT);
    PRINT_CONSTANT((guint) G_IO_PRI);
    PRINT_CONSTANT((gint) G_IO_STATUS_AGAIN);
    PRINT_CONSTANT((gint) G_IO_STATUS_EOF);
    PRINT_CONSTANT((gint) G_IO_STATUS_ERROR);
    PRINT_CONSTANT((gint) G_IO_STATUS_NORMAL);
    PRINT_CONSTANT(G_KEY_FILE_DESKTOP_GROUP);
    PRINT_CONSTANT(G_KEY_FILE_DESKTOP_KEY_ACTIONS);
    PRINT_CONSTANT(G_KEY_FILE_DESKTOP_KEY_CATEGORIES);
    PRINT_CONSTANT(G_KEY_FILE_DESKTOP_KEY_COMMENT);
    PRINT_CONSTANT(G_KEY_FILE_DESKTOP_KEY_DBUS_ACTIVATABLE);
    PRINT_CONSTANT(G_KEY_FILE_DESKTOP_KEY_EXEC);
    PRINT_CONSTANT(G_KEY_FILE_DESKTOP_KEY_GENERIC_NAME);
    PRINT_CONSTANT(G_KEY_FILE_DESKTOP_KEY_HIDDEN);
    PRINT_CONSTANT(G_KEY_FILE_DESKTOP_KEY_ICON);
    PRINT_CONSTANT(G_KEY_FILE_DESKTOP_KEY_MIME_TYPE);
    PRINT_CONSTANT(G_KEY_FILE_DESKTOP_KEY_NAME);
    PRINT_CONSTANT(G_KEY_FILE_DESKTOP_KEY_NOT_SHOW_IN);
    PRINT_CONSTANT(G_KEY_FILE_DESKTOP_KEY_NO_DISPLAY);
    PRINT_CONSTANT(G_KEY_FILE_DESKTOP_KEY_ONLY_SHOW_IN);
    PRINT_CONSTANT(G_KEY_FILE_DESKTOP_KEY_PATH);
    PRINT_CONSTANT(G_KEY_FILE_DESKTOP_KEY_STARTUP_NOTIFY);
    PRINT_CONSTANT(G_KEY_FILE_DESKTOP_KEY_STARTUP_WM_CLASS);
    PRINT_CONSTANT(G_KEY_FILE_DESKTOP_KEY_TERMINAL);
    PRINT_CONSTANT(G_KEY_FILE_DESKTOP_KEY_TRY_EXEC);
    PRINT_CONSTANT(G_KEY_FILE_DESKTOP_KEY_TYPE);
    PRINT_CONSTANT(G_KEY_FILE_DESKTOP_KEY_URL);
    PRINT_CONSTANT(G_KEY_FILE_DESKTOP_KEY_VERSION);
    PRINT_CONSTANT(G_KEY_FILE_DESKTOP_TYPE_APPLICATION);
    PRINT_CONSTANT(G_KEY_FILE_DESKTOP_TYPE_DIRECTORY);
    PRINT_CONSTANT(G_KEY_FILE_DESKTOP_TYPE_LINK);
    PRINT_CONSTANT((gint) G_KEY_FILE_ERROR_GROUP_NOT_FOUND);
    PRINT_CONSTANT((gint) G_KEY_FILE_ERROR_INVALID_VALUE);
    PRINT_CONSTANT((gint) G_KEY_FILE_ERROR_KEY_NOT_FOUND);
    PRINT_CONSTANT((gint) G_KEY_FILE_ERROR_NOT_FOUND);
    PRINT_CONSTANT((gint) G_KEY_FILE_ERROR_PARSE);
    PRINT_CONSTANT((gint) G_KEY_FILE_ERROR_UNKNOWN_ENCODING);
    PRINT_CONSTANT((guint) G_KEY_FILE_KEEP_COMMENTS);
    PRINT_CONSTANT((guint) G_KEY_FILE_KEEP_TRANSLATIONS);
    PRINT_CONSTANT((guint) G_KEY_FILE_NONE);
    PRINT_CONSTANT((gint) G_LEVEL_ORDER);
    PRINT_CONSTANT(G_LITTLE_ENDIAN);
    PRINT_CONSTANT(G_LN10);
    PRINT_CONSTANT(G_LN2);
    PRINT_CONSTANT(G_LOG_2_BASE_10);
    PRINT_CONSTANT(G_LOG_FATAL_MASK);
    PRINT_CONSTANT((guint) G_LOG_FLAG_FATAL);
    PRINT_CONSTANT((guint) G_LOG_FLAG_RECURSION);
    PRINT_CONSTANT((guint) G_LOG_LEVEL_CRITICAL);
    PRINT_CONSTANT((guint) G_LOG_LEVEL_DEBUG);
    PRINT_CONSTANT((guint) G_LOG_LEVEL_ERROR);
    PRINT_CONSTANT((guint) G_LOG_LEVEL_INFO);
    PRINT_CONSTANT((guint) G_LOG_LEVEL_MASK);
    PRINT_CONSTANT((guint) G_LOG_LEVEL_MESSAGE);
    PRINT_CONSTANT(G_LOG_LEVEL_USER_SHIFT);
    PRINT_CONSTANT((guint) G_LOG_LEVEL_WARNING);
    PRINT_CONSTANT((gint) G_LOG_WRITER_HANDLED);
    PRINT_CONSTANT((gint) G_LOG_WRITER_UNHANDLED);
    PRINT_CONSTANT((guint) G_MAIN_CONTEXT_FLAGS_NONE);
    PRINT_CONSTANT((guint) G_MAIN_CONTEXT_FLAGS_OWNERLESS_POLLING);
    PRINT_CONSTANT((guint) G_MARKUP_COLLECT_BOOLEAN);
    PRINT_CONSTANT((guint) G_MARKUP_COLLECT_INVALID);
    PRINT_CONSTANT((guint) G_MARKUP_COLLECT_OPTIONAL);
    PRINT_CONSTANT((guint) G_MARKUP_COLLECT_STRDUP);
    PRINT_CONSTANT((guint) G_MARKUP_COLLECT_STRING);
    PRINT_CONSTANT((guint) G_MARKUP_COLLECT_TRISTATE);
    PRINT_CONSTANT((guint) G_MARKUP_DEFAULT_FLAGS);
    PRINT_CONSTANT((guint) G_MARKUP_DO_NOT_USE_THIS_UNSUPPORTED_FLAG);
    PRINT_CONSTANT((gint) G_MARKUP_ERROR_BAD_UTF8);
    PRINT_CONSTANT((gint) G_MARKUP_ERROR_EMPTY);
    PRINT_CONSTANT((gint) G_MARKUP_ERROR_INVALID_CONTENT);
    PRINT_CONSTANT((gint) G_MARKUP_ERROR_MISSING_ATTRIBUTE);
    PRINT_CONSTANT((gint) G_MARKUP_ERROR_PARSE);
    PRINT_CONSTANT((gint) G_MARKUP_ERROR_UNKNOWN_ATTRIBUTE);
    PRINT_CONSTANT((gint) G_MARKUP_ERROR_UNKNOWN_ELEMENT);
    PRINT_CONSTANT((guint) G_MARKUP_IGNORE_QUALIFIED);
    PRINT_CONSTANT((guint) G_MARKUP_PREFIX_ERROR_POSITION);
    PRINT_CONSTANT((guint) G_MARKUP_TREAT_CDATA_AS_TEXT);
    PRINT_CONSTANT((gint) G_NORMALIZE_ALL);
    PRINT_CONSTANT((gint) G_NORMALIZE_ALL_COMPOSE);
    PRINT_CONSTANT((gint) G_NORMALIZE_DEFAULT);
    PRINT_CONSTANT((gint) G_NORMALIZE_DEFAULT_COMPOSE);
    PRINT_CONSTANT((gint) G_NORMALIZE_NFC);
    PRINT_CONSTANT((gint) G_NORMALIZE_NFD);
    PRINT_CONSTANT((gint) G_NORMALIZE_NFKC);
    PRINT_CONSTANT((gint) G_NORMALIZE_NFKD);
    PRINT_CONSTANT((gint) G_NUMBER_PARSER_ERROR_INVALID);
    PRINT_CONSTANT((gint) G_NUMBER_PARSER_ERROR_OUT_OF_BOUNDS);
    PRINT_CONSTANT((gint) G_ONCE_STATUS_NOTCALLED);
    PRINT_CONSTANT((gint) G_ONCE_STATUS_PROGRESS);
    PRINT_CONSTANT((gint) G_ONCE_STATUS_READY);
    PRINT_CONSTANT((gint) G_OPTION_ARG_CALLBACK);
    PRINT_CONSTANT((gint) G_OPTION_ARG_DOUBLE);
    PRINT_CONSTANT((gint) G_OPTION_ARG_FILENAME);
    PRINT_CONSTANT((gint) G_OPTION_ARG_FILENAME_ARRAY);
    PRINT_CONSTANT((gint) G_OPTION_ARG_INT);
    PRINT_CONSTANT((gint) G_OPTION_ARG_INT64);
    PRINT_CONSTANT((gint) G_OPTION_ARG_NONE);
    PRINT_CONSTANT((gint) G_OPTION_ARG_STRING);
    PRINT_CONSTANT((gint) G_OPTION_ARG_STRING_ARRAY);
    PRINT_CONSTANT((gint) G_OPTION_ERROR_BAD_VALUE);
    PRINT_CONSTANT((gint) G_OPTION_ERROR_FAILED);
    PRINT_CONSTANT((gint) G_OPTION_ERROR_UNKNOWN_OPTION);
    PRINT_CONSTANT((guint) G_OPTION_FLAG_FILENAME);
    PRINT_CONSTANT((guint) G_OPTION_FLAG_HIDDEN);
    PRINT_CONSTANT((guint) G_OPTION_FLAG_IN_MAIN);
    PRINT_CONSTANT((guint) G_OPTION_FLAG_NOALIAS);
    PRINT_CONSTANT((guint) G_OPTION_FLAG_NONE);
    PRINT_CONSTANT((guint) G_OPTION_FLAG_NO_ARG);
    PRINT_CONSTANT((guint) G_OPTION_FLAG_OPTIONAL_ARG);
    PRINT_CONSTANT((guint) G_OPTION_FLAG_REVERSE);
    PRINT_CONSTANT(G_OPTION_REMAINING);
    PRINT_CONSTANT(G_PDP_ENDIAN);
    PRINT_CONSTANT(G_PI);
    PRINT_CONSTANT(G_PI_2);
    PRINT_CONSTANT(G_PI_4);
    PRINT_CONSTANT((gint) G_POST_ORDER);
    PRINT_CONSTANT((gint) G_PRE_ORDER);
    PRINT_CONSTANT(G_PRIORITY_DEFAULT);
    PRINT_CONSTANT(G_PRIORITY_DEFAULT_IDLE);
    PRINT_CONSTANT(G_PRIORITY_HIGH);
    PRINT_CONSTANT(G_PRIORITY_HIGH_IDLE);
    PRINT_CONSTANT(G_PRIORITY_LOW);
    PRINT_CONSTANT((guint) G_REGEX_ANCHORED);
    PRINT_CONSTANT((guint) G_REGEX_BSR_ANYCRLF);
    PRINT_CONSTANT((guint) G_REGEX_CASELESS);
    PRINT_CONSTANT((guint) G_REGEX_DEFAULT);
    PRINT_CONSTANT((guint) G_REGEX_DOLLAR_ENDONLY);
    PRINT_CONSTANT((guint) G_REGEX_DOTALL);
    PRINT_CONSTANT((guint) G_REGEX_DUPNAMES);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_ASSERTION_EXPECTED);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_BACKTRACKING_CONTROL_VERB_ARGUMENT_FORBIDDEN);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_BACKTRACKING_CONTROL_VERB_ARGUMENT_REQUIRED);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_CHARACTER_VALUE_TOO_LARGE);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_COMPILE);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_DEFINE_REPETION);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_DUPLICATE_SUBPATTERN_NAME);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_EXPRESSION_TOO_LARGE);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_EXTRA_SUBPATTERN_NAME);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_HEX_CODE_TOO_LARGE);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_INCONSISTENT_NEWLINE_OPTIONS);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_INEXISTENT_SUBPATTERN_REFERENCE);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_INFINITE_LOOP);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_INTERNAL);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_INVALID_CONDITION);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_INVALID_CONTROL_CHAR);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_INVALID_DATA_CHARACTER);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_INVALID_ESCAPE_IN_CHARACTER_CLASS);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_INVALID_OCTAL_VALUE);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_INVALID_RELATIVE_REFERENCE);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_MALFORMED_CONDITION);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_MALFORMED_PROPERTY);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_MATCH);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_MEMORY_ERROR);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_MISSING_BACK_REFERENCE);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_MISSING_CONTROL_CHAR);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_MISSING_DIGIT);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_MISSING_NAME);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_MISSING_SUBPATTERN_NAME);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_MISSING_SUBPATTERN_NAME_TERMINATOR);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_NAME_TOO_LONG);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_NOTHING_TO_REPEAT);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_NOT_SUPPORTED_IN_CLASS);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_NUMBER_TOO_BIG);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_OPTIMIZE);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_POSIX_COLLATING_ELEMENTS_NOT_SUPPORTED);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_POSIX_NAMED_CLASS_OUTSIDE_CLASS);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_QUANTIFIERS_OUT_OF_ORDER);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_QUANTIFIER_TOO_BIG);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_RANGE_OUT_OF_ORDER);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_REPLACE);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_SINGLE_BYTE_MATCH_IN_LOOKBEHIND);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_STRAY_BACKSLASH);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_SUBPATTERN_NAME_TOO_LONG);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_TOO_MANY_BRANCHES_IN_DEFINE);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_TOO_MANY_CONDITIONAL_BRANCHES);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_TOO_MANY_FORWARD_REFERENCES);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_TOO_MANY_SUBPATTERNS);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_UNKNOWN_BACKTRACKING_CONTROL_VERB);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_UNKNOWN_POSIX_CLASS_NAME);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_UNKNOWN_PROPERTY);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_UNMATCHED_PARENTHESIS);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_UNRECOGNIZED_CHARACTER);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_UNRECOGNIZED_ESCAPE);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_UNTERMINATED_CHARACTER_CLASS);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_UNTERMINATED_COMMENT);
    PRINT_CONSTANT((gint) G_REGEX_ERROR_VARIABLE_LENGTH_LOOKBEHIND);
    PRINT_CONSTANT((guint) G_REGEX_EXTENDED);
    PRINT_CONSTANT((guint) G_REGEX_FIRSTLINE);
    PRINT_CONSTANT((guint) G_REGEX_JAVASCRIPT_COMPAT);
    PRINT_CONSTANT((guint) G_REGEX_MATCH_ANCHORED);
    PRINT_CONSTANT((guint) G_REGEX_MATCH_BSR_ANY);
    PRINT_CONSTANT((guint) G_REGEX_MATCH_BSR_ANYCRLF);
    PRINT_CONSTANT((guint) G_REGEX_MATCH_DEFAULT);
    PRINT_CONSTANT((guint) G_REGEX_MATCH_NEWLINE_ANY);
    PRINT_CONSTANT((guint) G_REGEX_MATCH_NEWLINE_ANYCRLF);
    PRINT_CONSTANT((guint) G_REGEX_MATCH_NEWLINE_CR);
    PRINT_CONSTANT((guint) G_REGEX_MATCH_NEWLINE_CRLF);
    PRINT_CONSTANT((guint) G_REGEX_MATCH_NEWLINE_LF);
    PRINT_CONSTANT((guint) G_REGEX_MATCH_NOTBOL);
    PRINT_CONSTANT((guint) G_REGEX_MATCH_NOTEMPTY);
    PRINT_CONSTANT((guint) G_REGEX_MATCH_NOTEMPTY_ATSTART);
    PRINT_CONSTANT((guint) G_REGEX_MATCH_NOTEOL);
    PRINT_CONSTANT((guint) G_REGEX_MATCH_PARTIAL);
    PRINT_CONSTANT((guint) G_REGEX_MATCH_PARTIAL_HARD);
    PRINT_CONSTANT((guint) G_REGEX_MATCH_PARTIAL_SOFT);
    PRINT_CONSTANT((guint) G_REGEX_MULTILINE);
    PRINT_CONSTANT((guint) G_REGEX_NEWLINE_ANYCRLF);
    PRINT_CONSTANT((guint) G_REGEX_NEWLINE_CR);
    PRINT_CONSTANT((guint) G_REGEX_NEWLINE_CRLF);
    PRINT_CONSTANT((guint) G_REGEX_NEWLINE_LF);
    PRINT_CONSTANT((guint) G_REGEX_NO_AUTO_CAPTURE);
    PRINT_CONSTANT((guint) G_REGEX_OPTIMIZE);
    PRINT_CONSTANT((guint) G_REGEX_RAW);
    PRINT_CONSTANT((guint) G_REGEX_UNGREEDY);
    PRINT_CONSTANT((gint) G_SEEK_CUR);
    PRINT_CONSTANT((gint) G_SEEK_END);
    PRINT_CONSTANT((gint) G_SEEK_SET);
    PRINT_CONSTANT((gint) G_SHELL_ERROR_BAD_QUOTING);
    PRINT_CONSTANT((gint) G_SHELL_ERROR_EMPTY_STRING);
    PRINT_CONSTANT((gint) G_SHELL_ERROR_FAILED);
    PRINT_CONSTANT((gint) G_SLICE_CONFIG_ALWAYS_MALLOC);
    PRINT_CONSTANT((gint) G_SLICE_CONFIG_BYPASS_MAGAZINES);
    PRINT_CONSTANT((gint) G_SLICE_CONFIG_CHUNK_SIZES);
    PRINT_CONSTANT((gint) G_SLICE_CONFIG_COLOR_INCREMENT);
    PRINT_CONSTANT((gint) G_SLICE_CONFIG_CONTENTION_COUNTER);
    PRINT_CONSTANT((gint) G_SLICE_CONFIG_WORKING_SET_MSECS);
    PRINT_CONSTANT(G_SOURCE_CONTINUE);
    PRINT_CONSTANT(G_SOURCE_REMOVE);
    PRINT_CONSTANT((guint) G_SPAWN_CHILD_INHERITS_STDERR);
    PRINT_CONSTANT((guint) G_SPAWN_CHILD_INHERITS_STDIN);
    PRINT_CONSTANT((guint) G_SPAWN_CHILD_INHERITS_STDOUT);
    PRINT_CONSTANT((guint) G_SPAWN_CLOEXEC_PIPES);
    PRINT_CONSTANT((guint) G_SPAWN_DEFAULT);
    PRINT_CONSTANT((guint) G_SPAWN_DO_NOT_REAP_CHILD);
    PRINT_CONSTANT((gint) G_SPAWN_ERROR_2BIG);
    PRINT_CONSTANT((gint) G_SPAWN_ERROR_ACCES);
    PRINT_CONSTANT((gint) G_SPAWN_ERROR_CHDIR);
    PRINT_CONSTANT((gint) G_SPAWN_ERROR_FAILED);
    PRINT_CONSTANT((gint) G_SPAWN_ERROR_FORK);
    PRINT_CONSTANT((gint) G_SPAWN_ERROR_INVAL);
    PRINT_CONSTANT((gint) G_SPAWN_ERROR_IO);
    PRINT_CONSTANT((gint) G_SPAWN_ERROR_ISDIR);
    PRINT_CONSTANT((gint) G_SPAWN_ERROR_LIBBAD);
    PRINT_CONSTANT((gint) G_SPAWN_ERROR_LOOP);
    PRINT_CONSTANT((gint) G_SPAWN_ERROR_MFILE);
    PRINT_CONSTANT((gint) G_SPAWN_ERROR_NAMETOOLONG);
    PRINT_CONSTANT((gint) G_SPAWN_ERROR_NFILE);
    PRINT_CONSTANT((gint) G_SPAWN_ERROR_NOENT);
    PRINT_CONSTANT((gint) G_SPAWN_ERROR_NOEXEC);
    PRINT_CONSTANT((gint) G_SPAWN_ERROR_NOMEM);
    PRINT_CONSTANT((gint) G_SPAWN_ERROR_NOTDIR);
    PRINT_CONSTANT((gint) G_SPAWN_ERROR_PERM);
    PRINT_CONSTANT((gint) G_SPAWN_ERROR_READ);
    PRINT_CONSTANT((gint) G_SPAWN_ERROR_TOO_BIG);
    PRINT_CONSTANT((gint) G_SPAWN_ERROR_TXTBUSY);
    PRINT_CONSTANT((guint) G_SPAWN_FILE_AND_ARGV_ZERO);
    PRINT_CONSTANT((guint) G_SPAWN_LEAVE_DESCRIPTORS_OPEN);
    PRINT_CONSTANT((guint) G_SPAWN_SEARCH_PATH);
    PRINT_CONSTANT((guint) G_SPAWN_SEARCH_PATH_FROM_ENVP);
    PRINT_CONSTANT((guint) G_SPAWN_STDERR_TO_DEV_NULL);
    PRINT_CONSTANT((guint) G_SPAWN_STDIN_FROM_DEV_NULL);
    PRINT_CONSTANT((guint) G_SPAWN_STDOUT_TO_DEV_NULL);
    PRINT_CONSTANT(G_SQRT2);
    PRINT_CONSTANT(G_STR_DELIMITERS);
    PRINT_CONSTANT((gint) G_TEST_BUILT);
    PRINT_CONSTANT((gint) G_TEST_DIST);
    PRINT_CONSTANT((gint) G_TEST_LOG_ERROR);
    PRINT_CONSTANT((gint) G_TEST_LOG_LIST_CASE);
    PRINT_CONSTANT((gint) G_TEST_LOG_MAX_RESULT);
    PRINT_CONSTANT((gint) G_TEST_LOG_MESSAGE);
    PRINT_CONSTANT((gint) G_TEST_LOG_MIN_RESULT);
    PRINT_CONSTANT((gint) G_TEST_LOG_NONE);
    PRINT_CONSTANT((gint) G_TEST_LOG_SKIP_CASE);
    PRINT_CONSTANT((gint) G_TEST_LOG_START_BINARY);
    PRINT_CONSTANT((gint) G_TEST_LOG_START_CASE);
    PRINT_CONSTANT((gint) G_TEST_LOG_START_SUITE);
    PRINT_CONSTANT((gint) G_TEST_LOG_STOP_CASE);
    PRINT_CONSTANT((gint) G_TEST_LOG_STOP_SUITE);
    PRINT_CONSTANT(G_TEST_OPTION_ISOLATE_DIRS);
    PRINT_CONSTANT((gint) G_TEST_RUN_FAILURE);
    PRINT_CONSTANT((gint) G_TEST_RUN_INCOMPLETE);
    PRINT_CONSTANT((gint) G_TEST_RUN_SKIPPED);
    PRINT_CONSTANT((gint) G_TEST_RUN_SUCCESS);
    PRINT_CONSTANT((guint) G_TEST_SUBPROCESS_DEFAULT);
    PRINT_CONSTANT((guint) G_TEST_SUBPROCESS_INHERIT_STDERR);
    PRINT_CONSTANT((guint) G_TEST_SUBPROCESS_INHERIT_STDIN);
    PRINT_CONSTANT((guint) G_TEST_SUBPROCESS_INHERIT_STDOUT);
    PRINT_CONSTANT((guint) G_TEST_TRAP_DEFAULT);
    PRINT_CONSTANT((guint) G_TEST_TRAP_INHERIT_STDIN);
    PRINT_CONSTANT((guint) G_TEST_TRAP_SILENCE_STDERR);
    PRINT_CONSTANT((guint) G_TEST_TRAP_SILENCE_STDOUT);
    PRINT_CONSTANT((gint) G_THREAD_ERROR_AGAIN);
    PRINT_CONSTANT(G_TIME_SPAN_DAY);
    PRINT_CONSTANT(G_TIME_SPAN_HOUR);
    PRINT_CONSTANT(G_TIME_SPAN_MILLISECOND);
    PRINT_CONSTANT(G_TIME_SPAN_MINUTE);
    PRINT_CONSTANT(G_TIME_SPAN_SECOND);
    PRINT_CONSTANT((gint) G_TIME_TYPE_DAYLIGHT);
    PRINT_CONSTANT((gint) G_TIME_TYPE_STANDARD);
    PRINT_CONSTANT((gint) G_TIME_TYPE_UNIVERSAL);
    PRINT_CONSTANT((gint) G_TOKEN_BINARY);
    PRINT_CONSTANT((gint) G_TOKEN_CHAR);
    PRINT_CONSTANT((gint) G_TOKEN_COMMA);
    PRINT_CONSTANT((gint) G_TOKEN_COMMENT_MULTI);
    PRINT_CONSTANT((gint) G_TOKEN_COMMENT_SINGLE);
    PRINT_CONSTANT((gint) G_TOKEN_EOF);
    PRINT_CONSTANT((gint) G_TOKEN_EQUAL_SIGN);
    PRINT_CONSTANT((gint) G_TOKEN_ERROR);
    PRINT_CONSTANT((gint) G_TOKEN_FLOAT);
    PRINT_CONSTANT((gint) G_TOKEN_HEX);
    PRINT_CONSTANT((gint) G_TOKEN_IDENTIFIER);
    PRINT_CONSTANT((gint) G_TOKEN_IDENTIFIER_NULL);
    PRINT_CONSTANT((gint) G_TOKEN_INT);
    PRINT_CONSTANT((gint) G_TOKEN_LEFT_BRACE);
    PRINT_CONSTANT((gint) G_TOKEN_LEFT_CURLY);
    PRINT_CONSTANT((gint) G_TOKEN_LEFT_PAREN);
    PRINT_CONSTANT((gint) G_TOKEN_NONE);
    PRINT_CONSTANT((gint) G_TOKEN_OCTAL);
    PRINT_CONSTANT((gint) G_TOKEN_RIGHT_BRACE);
    PRINT_CONSTANT((gint) G_TOKEN_RIGHT_CURLY);
    PRINT_CONSTANT((gint) G_TOKEN_RIGHT_PAREN);
    PRINT_CONSTANT((gint) G_TOKEN_STRING);
    PRINT_CONSTANT((gint) G_TOKEN_SYMBOL);
    PRINT_CONSTANT((guint) G_TRAVERSE_ALL);
    PRINT_CONSTANT((guint) G_TRAVERSE_LEAFS);
    PRINT_CONSTANT((guint) G_TRAVERSE_LEAVES);
    PRINT_CONSTANT((guint) G_TRAVERSE_MASK);
    PRINT_CONSTANT((guint) G_TRAVERSE_NON_LEAFS);
    PRINT_CONSTANT((guint) G_TRAVERSE_NON_LEAVES);
    PRINT_CONSTANT(G_UNICHAR_MAX_DECOMPOSITION_LENGTH);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_AFTER);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_ALPHABETIC);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_AMBIGUOUS);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_BEFORE);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_BEFORE_AND_AFTER);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_CARRIAGE_RETURN);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_CLOSE_PARANTHESIS);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_CLOSE_PARENTHESIS);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_CLOSE_PUNCTUATION);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_COMBINING_MARK);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_COMPLEX_CONTEXT);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_CONDITIONAL_JAPANESE_STARTER);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_CONTINGENT);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_EMOJI_BASE);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_EMOJI_MODIFIER);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_EXCLAMATION);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_HANGUL_LVT_SYLLABLE);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_HANGUL_LV_SYLLABLE);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_HANGUL_L_JAMO);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_HANGUL_T_JAMO);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_HANGUL_V_JAMO);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_HEBREW_LETTER);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_HYPHEN);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_IDEOGRAPHIC);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_INFIX_SEPARATOR);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_INSEPARABLE);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_LINE_FEED);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_MANDATORY);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_NEXT_LINE);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_NON_BREAKING_GLUE);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_NON_STARTER);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_NUMERIC);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_OPEN_PUNCTUATION);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_POSTFIX);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_PREFIX);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_QUOTATION);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_REGIONAL_INDICATOR);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_SPACE);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_SURROGATE);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_SYMBOL);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_UNKNOWN);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_WORD_JOINER);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_ZERO_WIDTH_JOINER);
    PRINT_CONSTANT((gint) G_UNICODE_BREAK_ZERO_WIDTH_SPACE);
    PRINT_CONSTANT((gint) G_UNICODE_CLOSE_PUNCTUATION);
    PRINT_CONSTANT((gint) G_UNICODE_CONNECT_PUNCTUATION);
    PRINT_CONSTANT((gint) G_UNICODE_CONTROL);
    PRINT_CONSTANT((gint) G_UNICODE_CURRENCY_SYMBOL);
    PRINT_CONSTANT((gint) G_UNICODE_DASH_PUNCTUATION);
    PRINT_CONSTANT((gint) G_UNICODE_DECIMAL_NUMBER);
    PRINT_CONSTANT((gint) G_UNICODE_ENCLOSING_MARK);
    PRINT_CONSTANT((gint) G_UNICODE_FINAL_PUNCTUATION);
    PRINT_CONSTANT((gint) G_UNICODE_FORMAT);
    PRINT_CONSTANT((gint) G_UNICODE_INITIAL_PUNCTUATION);
    PRINT_CONSTANT((gint) G_UNICODE_LETTER_NUMBER);
    PRINT_CONSTANT((gint) G_UNICODE_LINE_SEPARATOR);
    PRINT_CONSTANT((gint) G_UNICODE_LOWERCASE_LETTER);
    PRINT_CONSTANT((gint) G_UNICODE_MATH_SYMBOL);
    PRINT_CONSTANT((gint) G_UNICODE_MODIFIER_LETTER);
    PRINT_CONSTANT((gint) G_UNICODE_MODIFIER_SYMBOL);
    PRINT_CONSTANT((gint) G_UNICODE_NON_SPACING_MARK);
    PRINT_CONSTANT((gint) G_UNICODE_OPEN_PUNCTUATION);
    PRINT_CONSTANT((gint) G_UNICODE_OTHER_LETTER);
    PRINT_CONSTANT((gint) G_UNICODE_OTHER_NUMBER);
    PRINT_CONSTANT((gint) G_UNICODE_OTHER_PUNCTUATION);
    PRINT_CONSTANT((gint) G_UNICODE_OTHER_SYMBOL);
    PRINT_CONSTANT((gint) G_UNICODE_PARAGRAPH_SEPARATOR);
    PRINT_CONSTANT((gint) G_UNICODE_PRIVATE_USE);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_ADLAM);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_AHOM);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_ANATOLIAN_HIEROGLYPHS);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_ARABIC);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_ARMENIAN);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_AVESTAN);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_BALINESE);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_BAMUM);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_BASSA_VAH);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_BATAK);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_BENGALI);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_BHAIKSUKI);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_BOPOMOFO);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_BRAHMI);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_BRAILLE);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_BUGINESE);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_BUHID);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_CANADIAN_ABORIGINAL);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_CARIAN);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_CAUCASIAN_ALBANIAN);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_CHAKMA);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_CHAM);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_CHEROKEE);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_CHORASMIAN);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_COMMON);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_COPTIC);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_CUNEIFORM);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_CYPRIOT);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_CYPRO_MINOAN);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_CYRILLIC);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_DESERET);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_DEVANAGARI);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_DIVES_AKURU);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_DOGRA);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_DUPLOYAN);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_EGYPTIAN_HIEROGLYPHS);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_ELBASAN);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_ELYMAIC);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_ETHIOPIC);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_GEORGIAN);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_GLAGOLITIC);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_GOTHIC);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_GRANTHA);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_GREEK);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_GUJARATI);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_GUNJALA_GONDI);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_GURMUKHI);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_HAN);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_HANGUL);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_HANIFI_ROHINGYA);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_HANUNOO);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_HATRAN);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_HEBREW);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_HIRAGANA);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_IMPERIAL_ARAMAIC);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_INHERITED);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_INSCRIPTIONAL_PAHLAVI);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_INSCRIPTIONAL_PARTHIAN);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_INVALID_CODE);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_JAVANESE);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_KAITHI);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_KANNADA);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_KATAKANA);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_KAWI);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_KAYAH_LI);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_KHAROSHTHI);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_KHITAN_SMALL_SCRIPT);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_KHMER);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_KHOJKI);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_KHUDAWADI);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_LAO);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_LATIN);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_LEPCHA);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_LIMBU);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_LINEAR_A);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_LINEAR_B);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_LISU);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_LYCIAN);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_LYDIAN);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_MAHAJANI);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_MAKASAR);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_MALAYALAM);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_MANDAIC);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_MANICHAEAN);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_MARCHEN);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_MASARAM_GONDI);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_MATH);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_MEDEFAIDRIN);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_MEETEI_MAYEK);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_MENDE_KIKAKUI);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_MEROITIC_CURSIVE);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_MEROITIC_HIEROGLYPHS);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_MIAO);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_MODI);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_MONGOLIAN);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_MRO);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_MULTANI);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_MYANMAR);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_NABATAEAN);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_NAG_MUNDARI);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_NANDINAGARI);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_NEWA);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_NEW_TAI_LUE);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_NKO);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_NUSHU);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_NYIAKENG_PUACHUE_HMONG);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_OGHAM);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_OLD_HUNGARIAN);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_OLD_ITALIC);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_OLD_NORTH_ARABIAN);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_OLD_PERMIC);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_OLD_PERSIAN);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_OLD_SOGDIAN);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_OLD_SOUTH_ARABIAN);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_OLD_TURKIC);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_OLD_UYGHUR);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_OL_CHIKI);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_ORIYA);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_OSAGE);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_OSMANYA);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_PAHAWH_HMONG);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_PALMYRENE);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_PAU_CIN_HAU);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_PHAGS_PA);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_PHOENICIAN);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_PSALTER_PAHLAVI);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_REJANG);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_RUNIC);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_SAMARITAN);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_SAURASHTRA);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_SHARADA);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_SHAVIAN);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_SIDDHAM);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_SIGNWRITING);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_SINHALA);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_SOGDIAN);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_SORA_SOMPENG);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_SOYOMBO);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_SUNDANESE);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_SYLOTI_NAGRI);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_SYRIAC);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_TAGALOG);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_TAGBANWA);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_TAI_LE);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_TAI_THAM);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_TAI_VIET);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_TAKRI);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_TAMIL);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_TANGSA);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_TANGUT);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_TELUGU);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_THAANA);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_THAI);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_TIBETAN);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_TIFINAGH);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_TIRHUTA);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_TOTO);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_UGARITIC);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_UNKNOWN);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_VAI);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_VITHKUQI);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_WANCHO);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_WARANG_CITI);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_YEZIDI);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_YI);
    PRINT_CONSTANT((gint) G_UNICODE_SCRIPT_ZANABAZAR_SQUARE);
    PRINT_CONSTANT((gint) G_UNICODE_SPACE_SEPARATOR);
    PRINT_CONSTANT((gint) G_UNICODE_SPACING_MARK);
    PRINT_CONSTANT((gint) G_UNICODE_SURROGATE);
    PRINT_CONSTANT((gint) G_UNICODE_TITLECASE_LETTER);
    PRINT_CONSTANT((gint) G_UNICODE_UNASSIGNED);
    PRINT_CONSTANT((gint) G_UNICODE_UPPERCASE_LETTER);
    PRINT_CONSTANT((gint) G_URI_ERROR_BAD_AUTH_PARAMS);
    PRINT_CONSTANT((gint) G_URI_ERROR_BAD_FRAGMENT);
    PRINT_CONSTANT((gint) G_URI_ERROR_BAD_HOST);
    PRINT_CONSTANT((gint) G_URI_ERROR_BAD_PASSWORD);
    PRINT_CONSTANT((gint) G_URI_ERROR_BAD_PATH);
    PRINT_CONSTANT((gint) G_URI_ERROR_BAD_PORT);
    PRINT_CONSTANT((gint) G_URI_ERROR_BAD_QUERY);
    PRINT_CONSTANT((gint) G_URI_ERROR_BAD_SCHEME);
    PRINT_CONSTANT((gint) G_URI_ERROR_BAD_USER);
    PRINT_CONSTANT((gint) G_URI_ERROR_FAILED);
    PRINT_CONSTANT((guint) G_URI_FLAGS_ENCODED);
    PRINT_CONSTANT((guint) G_URI_FLAGS_ENCODED_FRAGMENT);
    PRINT_CONSTANT((guint) G_URI_FLAGS_ENCODED_PATH);
    PRINT_CONSTANT((guint) G_URI_FLAGS_ENCODED_QUERY);
    PRINT_CONSTANT((guint) G_URI_FLAGS_HAS_AUTH_PARAMS);
    PRINT_CONSTANT((guint) G_URI_FLAGS_HAS_PASSWORD);
    PRINT_CONSTANT((guint) G_URI_FLAGS_NONE);
    PRINT_CONSTANT((guint) G_URI_FLAGS_NON_DNS);
    PRINT_CONSTANT((guint) G_URI_FLAGS_PARSE_RELAXED);
    PRINT_CONSTANT((guint) G_URI_FLAGS_SCHEME_NORMALIZE);
    PRINT_CONSTANT((guint) G_URI_HIDE_AUTH_PARAMS);
    PRINT_CONSTANT((guint) G_URI_HIDE_FRAGMENT);
    PRINT_CONSTANT((guint) G_URI_HIDE_NONE);
    PRINT_CONSTANT((guint) G_URI_HIDE_PASSWORD);
    PRINT_CONSTANT((guint) G_URI_HIDE_QUERY);
    PRINT_CONSTANT((guint) G_URI_HIDE_USERINFO);
    PRINT_CONSTANT((guint) G_URI_PARAMS_CASE_INSENSITIVE);
    PRINT_CONSTANT((guint) G_URI_PARAMS_NONE);
    PRINT_CONSTANT((guint) G_URI_PARAMS_PARSE_RELAXED);
    PRINT_CONSTANT((guint) G_URI_PARAMS_WWW_FORM);
    PRINT_CONSTANT(G_URI_RESERVED_CHARS_GENERIC_DELIMITERS);
    PRINT_CONSTANT(G_URI_RESERVED_CHARS_SUBCOMPONENT_DELIMITERS);
    PRINT_CONSTANT(G_USEC_PER_SEC);
    PRINT_CONSTANT((gint) G_USER_DIRECTORY_DESKTOP);
    PRINT_CONSTANT((gint) G_USER_DIRECTORY_DOCUMENTS);
    PRINT_CONSTANT((gint) G_USER_DIRECTORY_DOWNLOAD);
    PRINT_CONSTANT((gint) G_USER_DIRECTORY_MUSIC);
    PRINT_CONSTANT((gint) G_USER_DIRECTORY_PICTURES);
    PRINT_CONSTANT((gint) G_USER_DIRECTORY_PUBLIC_SHARE);
    PRINT_CONSTANT((gint) G_USER_DIRECTORY_TEMPLATES);
    PRINT_CONSTANT((gint) G_USER_DIRECTORY_VIDEOS);
    PRINT_CONSTANT((gint) G_USER_N_DIRECTORIES);
    PRINT_CONSTANT((gint) G_VARIANT_CLASS_ARRAY);
    PRINT_CONSTANT((gint) G_VARIANT_CLASS_BOOLEAN);
    PRINT_CONSTANT((gint) G_VARIANT_CLASS_BYTE);
    PRINT_CONSTANT((gint) G_VARIANT_CLASS_DICT_ENTRY);
    PRINT_CONSTANT((gint) G_VARIANT_CLASS_DOUBLE);
    PRINT_CONSTANT((gint) G_VARIANT_CLASS_HANDLE);
    PRINT_CONSTANT((gint) G_VARIANT_CLASS_INT16);
    PRINT_CONSTANT((gint) G_VARIANT_CLASS_INT32);
    PRINT_CONSTANT((gint) G_VARIANT_CLASS_INT64);
    PRINT_CONSTANT((gint) G_VARIANT_CLASS_MAYBE);
    PRINT_CONSTANT((gint) G_VARIANT_CLASS_OBJECT_PATH);
    PRINT_CONSTANT((gint) G_VARIANT_CLASS_SIGNATURE);
    PRINT_CONSTANT((gint) G_VARIANT_CLASS_STRING);
    PRINT_CONSTANT((gint) G_VARIANT_CLASS_TUPLE);
    PRINT_CONSTANT((gint) G_VARIANT_CLASS_UINT16);
    PRINT_CONSTANT((gint) G_VARIANT_CLASS_UINT32);
    PRINT_CONSTANT((gint) G_VARIANT_CLASS_UINT64);
    PRINT_CONSTANT((gint) G_VARIANT_CLASS_VARIANT);
    PRINT_CONSTANT((gint) G_VARIANT_PARSE_ERROR_BASIC_TYPE_EXPECTED);
    PRINT_CONSTANT((gint) G_VARIANT_PARSE_ERROR_CANNOT_INFER_TYPE);
    PRINT_CONSTANT((gint) G_VARIANT_PARSE_ERROR_DEFINITE_TYPE_EXPECTED);
    PRINT_CONSTANT((gint) G_VARIANT_PARSE_ERROR_FAILED);
    PRINT_CONSTANT((gint) G_VARIANT_PARSE_ERROR_INPUT_NOT_AT_END);
    PRINT_CONSTANT((gint) G_VARIANT_PARSE_ERROR_INVALID_CHARACTER);
    PRINT_CONSTANT((gint) G_VARIANT_PARSE_ERROR_INVALID_FORMAT_STRING);
    PRINT_CONSTANT((gint) G_VARIANT_PARSE_ERROR_INVALID_OBJECT_PATH);
    PRINT_CONSTANT((gint) G_VARIANT_PARSE_ERROR_INVALID_SIGNATURE);
    PRINT_CONSTANT((gint) G_VARIANT_PARSE_ERROR_INVALID_TYPE_STRING);
    PRINT_CONSTANT((gint) G_VARIANT_PARSE_ERROR_NO_COMMON_TYPE);
    PRINT_CONSTANT((gint) G_VARIANT_PARSE_ERROR_NUMBER_OUT_OF_RANGE);
    PRINT_CONSTANT((gint) G_VARIANT_PARSE_ERROR_NUMBER_TOO_BIG);
    PRINT_CONSTANT((gint) G_VARIANT_PARSE_ERROR_RECURSION);
    PRINT_CONSTANT((gint) G_VARIANT_PARSE_ERROR_TYPE_ERROR);
    PRINT_CONSTANT((gint) G_VARIANT_PARSE_ERROR_UNEXPECTED_TOKEN);
    PRINT_CONSTANT((gint) G_VARIANT_PARSE_ERROR_UNKNOWN_KEYWORD);
    PRINT_CONSTANT((gint) G_VARIANT_PARSE_ERROR_UNTERMINATED_STRING_CONSTANT);
    PRINT_CONSTANT((gint) G_VARIANT_PARSE_ERROR_VALUE_EXPECTED);
    PRINT_CONSTANT(G_WIN32_MSG_HANDLE);
    return 0;
}
