// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

use glib::translate::*;

glib::wrapper! {
    #[derive(Debug, PartialEq, Eq, PartialOrd, Ord, Hash)]
    pub struct ContentFormatsBuilder(Shared<ffi::GdkContentFormatsBuilder>);

    match fn {
        ref => |ptr| ffi::gdk_content_formats_builder_ref(ptr),
        unref => |ptr| ffi::gdk_content_formats_builder_unref(ptr),
        type_ => || ffi::gdk_content_formats_builder_get_type(),
    }
}

impl ContentFormatsBuilder {
    #[doc(alias = "gdk_content_formats_builder_new")]
    pub fn new() -> ContentFormatsBuilder {
        assert_initialized_main_thread!();
        unsafe { from_glib_full(ffi::gdk_content_formats_builder_new()) }
    }
}

impl Default for ContentFormatsBuilder {
    fn default() -> Self {
        Self::new()
    }
}
