// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

#include "manual.h"
#include <stdio.h>

#define PRINT_CONSTANT(CONSTANT_NAME) \
    printf("%s;", #CONSTANT_NAME); \
    printf(_Generic((CONSTANT_NAME), \
                    char *: "%s", \
                    const char *: "%s", \
                    char: "%c", \
                    signed char: "%hhd", \
                    unsigned char: "%hhu", \
                    short int: "%hd", \
                    unsigned short int: "%hu", \
                    int: "%d", \
                    unsigned int: "%u", \
                    long: "%ld", \
                    unsigned long: "%lu", \
                    long long: "%lld", \
                    unsigned long long: "%llu", \
                    float: "%f", \
                    double: "%f", \
                    long double: "%ld"), \
           CONSTANT_NAME); \
    printf("\n");

int main() {
    PRINT_CONSTANT((gint) GDK_COLORSPACE_RGB);
    PRINT_CONSTANT((gint) GDK_INTERP_BILINEAR);
    PRINT_CONSTANT((gint) GDK_INTERP_HYPER);
    PRINT_CONSTANT((gint) GDK_INTERP_NEAREST);
    PRINT_CONSTANT((gint) GDK_INTERP_TILES);
    PRINT_CONSTANT((gint) GDK_PIXBUF_ALPHA_BILEVEL);
    PRINT_CONSTANT((gint) GDK_PIXBUF_ALPHA_FULL);
    PRINT_CONSTANT((gint) GDK_PIXBUF_ERROR_BAD_OPTION);
    PRINT_CONSTANT((gint) GDK_PIXBUF_ERROR_CORRUPT_IMAGE);
    PRINT_CONSTANT((gint) GDK_PIXBUF_ERROR_FAILED);
    PRINT_CONSTANT((gint) GDK_PIXBUF_ERROR_INCOMPLETE_ANIMATION);
    PRINT_CONSTANT((gint) GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY);
    PRINT_CONSTANT((gint) GDK_PIXBUF_ERROR_UNKNOWN_TYPE);
    PRINT_CONSTANT((gint) GDK_PIXBUF_ERROR_UNSUPPORTED_OPERATION);
    PRINT_CONSTANT((guint) GDK_PIXBUF_FORMAT_SCALABLE);
    PRINT_CONSTANT((guint) GDK_PIXBUF_FORMAT_THREADSAFE);
    PRINT_CONSTANT((guint) GDK_PIXBUF_FORMAT_WRITABLE);
    PRINT_CONSTANT((gint) GDK_PIXBUF_ROTATE_CLOCKWISE);
    PRINT_CONSTANT((gint) GDK_PIXBUF_ROTATE_COUNTERCLOCKWISE);
    PRINT_CONSTANT((gint) GDK_PIXBUF_ROTATE_NONE);
    PRINT_CONSTANT((gint) GDK_PIXBUF_ROTATE_UPSIDEDOWN);
    return 0;
}
