// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

#include "manual.h"
#include <stdalign.h>
#include <stdio.h>

int main() {
    printf("%s;%zu;%zu\n", "GtkAccessibleAutocomplete", sizeof(GtkAccessibleAutocomplete), alignof(GtkAccessibleAutocomplete));
    printf("%s;%zu;%zu\n", "GtkAccessibleInterface", sizeof(GtkAccessibleInterface), alignof(GtkAccessibleInterface));
    printf("%s;%zu;%zu\n", "GtkAccessibleInvalidState", sizeof(GtkAccessibleInvalidState), alignof(GtkAccessibleInvalidState));
    printf("%s;%zu;%zu\n", "GtkAccessiblePlatformState", sizeof(GtkAccessiblePlatformState), alignof(GtkAccessiblePlatformState));
    printf("%s;%zu;%zu\n", "GtkAccessibleProperty", sizeof(GtkAccessibleProperty), alignof(GtkAccessibleProperty));
    printf("%s;%zu;%zu\n", "GtkAccessibleRangeInterface", sizeof(GtkAccessibleRangeInterface), alignof(GtkAccessibleRangeInterface));
    printf("%s;%zu;%zu\n", "GtkAccessibleRelation", sizeof(GtkAccessibleRelation), alignof(GtkAccessibleRelation));
    printf("%s;%zu;%zu\n", "GtkAccessibleRole", sizeof(GtkAccessibleRole), alignof(GtkAccessibleRole));
    printf("%s;%zu;%zu\n", "GtkAccessibleSort", sizeof(GtkAccessibleSort), alignof(GtkAccessibleSort));
    printf("%s;%zu;%zu\n", "GtkAccessibleState", sizeof(GtkAccessibleState), alignof(GtkAccessibleState));
    printf("%s;%zu;%zu\n", "GtkAccessibleTristate", sizeof(GtkAccessibleTristate), alignof(GtkAccessibleTristate));
    printf("%s;%zu;%zu\n", "GtkActionableInterface", sizeof(GtkActionableInterface), alignof(GtkActionableInterface));
    printf("%s;%zu;%zu\n", "GtkAdjustment", sizeof(GtkAdjustment), alignof(GtkAdjustment));
    printf("%s;%zu;%zu\n", "GtkAdjustmentClass", sizeof(GtkAdjustmentClass), alignof(GtkAdjustmentClass));
    printf("%s;%zu;%zu\n", "GtkAlertDialogClass", sizeof(GtkAlertDialogClass), alignof(GtkAlertDialogClass));
    printf("%s;%zu;%zu\n", "GtkAlign", sizeof(GtkAlign), alignof(GtkAlign));
    printf("%s;%zu;%zu\n", "GtkAllocation", sizeof(GtkAllocation), alignof(GtkAllocation));
    printf("%s;%zu;%zu\n", "GtkApplication", sizeof(GtkApplication), alignof(GtkApplication));
    printf("%s;%zu;%zu\n", "GtkApplicationClass", sizeof(GtkApplicationClass), alignof(GtkApplicationClass));
    printf("%s;%zu;%zu\n", "GtkApplicationInhibitFlags", sizeof(GtkApplicationInhibitFlags), alignof(GtkApplicationInhibitFlags));
    printf("%s;%zu;%zu\n", "GtkApplicationWindow", sizeof(GtkApplicationWindow), alignof(GtkApplicationWindow));
    printf("%s;%zu;%zu\n", "GtkApplicationWindowClass", sizeof(GtkApplicationWindowClass), alignof(GtkApplicationWindowClass));
    printf("%s;%zu;%zu\n", "GtkArrowType", sizeof(GtkArrowType), alignof(GtkArrowType));
    printf("%s;%zu;%zu\n", "GtkAssistantPageType", sizeof(GtkAssistantPageType), alignof(GtkAssistantPageType));
    printf("%s;%zu;%zu\n", "GtkBaselinePosition", sizeof(GtkBaselinePosition), alignof(GtkBaselinePosition));
    printf("%s;%zu;%zu\n", "GtkBinLayoutClass", sizeof(GtkBinLayoutClass), alignof(GtkBinLayoutClass));
    printf("%s;%zu;%zu\n", "GtkBitsetIter", sizeof(GtkBitsetIter), alignof(GtkBitsetIter));
    printf("%s;%zu;%zu\n", "GtkBookmarkListClass", sizeof(GtkBookmarkListClass), alignof(GtkBookmarkListClass));
    printf("%s;%zu;%zu\n", "GtkBoolFilterClass", sizeof(GtkBoolFilterClass), alignof(GtkBoolFilterClass));
    printf("%s;%zu;%zu\n", "GtkBorder", sizeof(GtkBorder), alignof(GtkBorder));
    printf("%s;%zu;%zu\n", "GtkBorderStyle", sizeof(GtkBorderStyle), alignof(GtkBorderStyle));
    printf("%s;%zu;%zu\n", "GtkBox", sizeof(GtkBox), alignof(GtkBox));
    printf("%s;%zu;%zu\n", "GtkBoxClass", sizeof(GtkBoxClass), alignof(GtkBoxClass));
    printf("%s;%zu;%zu\n", "GtkBoxLayoutClass", sizeof(GtkBoxLayoutClass), alignof(GtkBoxLayoutClass));
    printf("%s;%zu;%zu\n", "GtkBuildableIface", sizeof(GtkBuildableIface), alignof(GtkBuildableIface));
    printf("%s;%zu;%zu\n", "GtkBuildableParser", sizeof(GtkBuildableParser), alignof(GtkBuildableParser));
    printf("%s;%zu;%zu\n", "GtkBuilderCScope", sizeof(GtkBuilderCScope), alignof(GtkBuilderCScope));
    printf("%s;%zu;%zu\n", "GtkBuilderCScopeClass", sizeof(GtkBuilderCScopeClass), alignof(GtkBuilderCScopeClass));
    printf("%s;%zu;%zu\n", "GtkBuilderClosureFlags", sizeof(GtkBuilderClosureFlags), alignof(GtkBuilderClosureFlags));
    printf("%s;%zu;%zu\n", "GtkBuilderError", sizeof(GtkBuilderError), alignof(GtkBuilderError));
    printf("%s;%zu;%zu\n", "GtkBuilderScopeInterface", sizeof(GtkBuilderScopeInterface), alignof(GtkBuilderScopeInterface));
    printf("%s;%zu;%zu\n", "GtkButton", sizeof(GtkButton), alignof(GtkButton));
    printf("%s;%zu;%zu\n", "GtkButtonClass", sizeof(GtkButtonClass), alignof(GtkButtonClass));
    printf("%s;%zu;%zu\n", "GtkButtonsType", sizeof(GtkButtonsType), alignof(GtkButtonsType));
    printf("%s;%zu;%zu\n", "GtkCellArea", sizeof(GtkCellArea), alignof(GtkCellArea));
    printf("%s;%zu;%zu\n", "GtkCellAreaClass", sizeof(GtkCellAreaClass), alignof(GtkCellAreaClass));
    printf("%s;%zu;%zu\n", "GtkCellAreaContext", sizeof(GtkCellAreaContext), alignof(GtkCellAreaContext));
    printf("%s;%zu;%zu\n", "GtkCellAreaContextClass", sizeof(GtkCellAreaContextClass), alignof(GtkCellAreaContextClass));
    printf("%s;%zu;%zu\n", "GtkCellEditableIface", sizeof(GtkCellEditableIface), alignof(GtkCellEditableIface));
    printf("%s;%zu;%zu\n", "GtkCellLayoutIface", sizeof(GtkCellLayoutIface), alignof(GtkCellLayoutIface));
    printf("%s;%zu;%zu\n", "GtkCellRenderer", sizeof(GtkCellRenderer), alignof(GtkCellRenderer));
    printf("%s;%zu;%zu\n", "GtkCellRendererAccelMode", sizeof(GtkCellRendererAccelMode), alignof(GtkCellRendererAccelMode));
    printf("%s;%zu;%zu\n", "GtkCellRendererClass", sizeof(GtkCellRendererClass), alignof(GtkCellRendererClass));
    printf("%s;%zu;%zu\n", "GtkCellRendererMode", sizeof(GtkCellRendererMode), alignof(GtkCellRendererMode));
    printf("%s;%zu;%zu\n", "GtkCellRendererState", sizeof(GtkCellRendererState), alignof(GtkCellRendererState));
    printf("%s;%zu;%zu\n", "GtkCellRendererText", sizeof(GtkCellRendererText), alignof(GtkCellRendererText));
    printf("%s;%zu;%zu\n", "GtkCellRendererTextClass", sizeof(GtkCellRendererTextClass), alignof(GtkCellRendererTextClass));
    printf("%s;%zu;%zu\n", "GtkCenterLayoutClass", sizeof(GtkCenterLayoutClass), alignof(GtkCenterLayoutClass));
    printf("%s;%zu;%zu\n", "GtkCheckButton", sizeof(GtkCheckButton), alignof(GtkCheckButton));
    printf("%s;%zu;%zu\n", "GtkCheckButtonClass", sizeof(GtkCheckButtonClass), alignof(GtkCheckButtonClass));
    printf("%s;%zu;%zu\n", "GtkCollation", sizeof(GtkCollation), alignof(GtkCollation));
    printf("%s;%zu;%zu\n", "GtkColorChooserInterface", sizeof(GtkColorChooserInterface), alignof(GtkColorChooserInterface));
    printf("%s;%zu;%zu\n", "GtkColorDialogButtonClass", sizeof(GtkColorDialogButtonClass), alignof(GtkColorDialogButtonClass));
    printf("%s;%zu;%zu\n", "GtkColorDialogClass", sizeof(GtkColorDialogClass), alignof(GtkColorDialogClass));
    printf("%s;%zu;%zu\n", "GtkColumnViewSorterClass", sizeof(GtkColumnViewSorterClass), alignof(GtkColumnViewSorterClass));
    printf("%s;%zu;%zu\n", "GtkComboBox", sizeof(GtkComboBox), alignof(GtkComboBox));
    printf("%s;%zu;%zu\n", "GtkComboBoxClass", sizeof(GtkComboBoxClass), alignof(GtkComboBoxClass));
    printf("%s;%zu;%zu\n", "GtkConstraintAttribute", sizeof(GtkConstraintAttribute), alignof(GtkConstraintAttribute));
    printf("%s;%zu;%zu\n", "GtkConstraintClass", sizeof(GtkConstraintClass), alignof(GtkConstraintClass));
    printf("%s;%zu;%zu\n", "GtkConstraintGuideClass", sizeof(GtkConstraintGuideClass), alignof(GtkConstraintGuideClass));
    printf("%s;%zu;%zu\n", "GtkConstraintLayoutChildClass", sizeof(GtkConstraintLayoutChildClass), alignof(GtkConstraintLayoutChildClass));
    printf("%s;%zu;%zu\n", "GtkConstraintLayoutClass", sizeof(GtkConstraintLayoutClass), alignof(GtkConstraintLayoutClass));
    printf("%s;%zu;%zu\n", "GtkConstraintRelation", sizeof(GtkConstraintRelation), alignof(GtkConstraintRelation));
    printf("%s;%zu;%zu\n", "GtkConstraintStrength", sizeof(GtkConstraintStrength), alignof(GtkConstraintStrength));
    printf("%s;%zu;%zu\n", "GtkConstraintVflParserError", sizeof(GtkConstraintVflParserError), alignof(GtkConstraintVflParserError));
    printf("%s;%zu;%zu\n", "GtkContentFit", sizeof(GtkContentFit), alignof(GtkContentFit));
    printf("%s;%zu;%zu\n", "GtkCornerType", sizeof(GtkCornerType), alignof(GtkCornerType));
    printf("%s;%zu;%zu\n", "GtkCssLocation", sizeof(GtkCssLocation), alignof(GtkCssLocation));
    printf("%s;%zu;%zu\n", "GtkCssParserError", sizeof(GtkCssParserError), alignof(GtkCssParserError));
    printf("%s;%zu;%zu\n", "GtkCssParserWarning", sizeof(GtkCssParserWarning), alignof(GtkCssParserWarning));
    printf("%s;%zu;%zu\n", "GtkCssProvider", sizeof(GtkCssProvider), alignof(GtkCssProvider));
    printf("%s;%zu;%zu\n", "GtkCustomFilterClass", sizeof(GtkCustomFilterClass), alignof(GtkCustomFilterClass));
    printf("%s;%zu;%zu\n", "GtkCustomLayoutClass", sizeof(GtkCustomLayoutClass), alignof(GtkCustomLayoutClass));
    printf("%s;%zu;%zu\n", "GtkCustomSorterClass", sizeof(GtkCustomSorterClass), alignof(GtkCustomSorterClass));
    printf("%s;%zu;%zu\n", "GtkDebugFlags", sizeof(GtkDebugFlags), alignof(GtkDebugFlags));
    printf("%s;%zu;%zu\n", "GtkDeleteType", sizeof(GtkDeleteType), alignof(GtkDeleteType));
    printf("%s;%zu;%zu\n", "GtkDialog", sizeof(GtkDialog), alignof(GtkDialog));
    printf("%s;%zu;%zu\n", "GtkDialogClass", sizeof(GtkDialogClass), alignof(GtkDialogClass));
    printf("%s;%zu;%zu\n", "GtkDialogError", sizeof(GtkDialogError), alignof(GtkDialogError));
    printf("%s;%zu;%zu\n", "GtkDialogFlags", sizeof(GtkDialogFlags), alignof(GtkDialogFlags));
    printf("%s;%zu;%zu\n", "GtkDirectionType", sizeof(GtkDirectionType), alignof(GtkDirectionType));
    printf("%s;%zu;%zu\n", "GtkDirectoryListClass", sizeof(GtkDirectoryListClass), alignof(GtkDirectoryListClass));
    printf("%s;%zu;%zu\n", "GtkDragIconClass", sizeof(GtkDragIconClass), alignof(GtkDragIconClass));
    printf("%s;%zu;%zu\n", "GtkDrawingArea", sizeof(GtkDrawingArea), alignof(GtkDrawingArea));
    printf("%s;%zu;%zu\n", "GtkDrawingAreaClass", sizeof(GtkDrawingAreaClass), alignof(GtkDrawingAreaClass));
    printf("%s;%zu;%zu\n", "GtkDropDownClass", sizeof(GtkDropDownClass), alignof(GtkDropDownClass));
    printf("%s;%zu;%zu\n", "GtkEditableInterface", sizeof(GtkEditableInterface), alignof(GtkEditableInterface));
    printf("%s;%zu;%zu\n", "GtkEditableLabelClass", sizeof(GtkEditableLabelClass), alignof(GtkEditableLabelClass));
    printf("%s;%zu;%zu\n", "GtkEditableProperties", sizeof(GtkEditableProperties), alignof(GtkEditableProperties));
    printf("%s;%zu;%zu\n", "GtkEntry", sizeof(GtkEntry), alignof(GtkEntry));
    printf("%s;%zu;%zu\n", "GtkEntryBuffer", sizeof(GtkEntryBuffer), alignof(GtkEntryBuffer));
    printf("%s;%zu;%zu\n", "GtkEntryBufferClass", sizeof(GtkEntryBufferClass), alignof(GtkEntryBufferClass));
    printf("%s;%zu;%zu\n", "GtkEntryClass", sizeof(GtkEntryClass), alignof(GtkEntryClass));
    printf("%s;%zu;%zu\n", "GtkEntryIconPosition", sizeof(GtkEntryIconPosition), alignof(GtkEntryIconPosition));
    printf("%s;%zu;%zu\n", "GtkEventControllerScrollFlags", sizeof(GtkEventControllerScrollFlags), alignof(GtkEventControllerScrollFlags));
    printf("%s;%zu;%zu\n", "GtkEventSequenceState", sizeof(GtkEventSequenceState), alignof(GtkEventSequenceState));
    printf("%s;%zu;%zu\n", "GtkFileChooserAction", sizeof(GtkFileChooserAction), alignof(GtkFileChooserAction));
    printf("%s;%zu;%zu\n", "GtkFileChooserError", sizeof(GtkFileChooserError), alignof(GtkFileChooserError));
    printf("%s;%zu;%zu\n", "GtkFileChooserNativeClass", sizeof(GtkFileChooserNativeClass), alignof(GtkFileChooserNativeClass));
    printf("%s;%zu;%zu\n", "GtkFileDialogClass", sizeof(GtkFileDialogClass), alignof(GtkFileDialogClass));
    printf("%s;%zu;%zu\n", "GtkFileLauncherClass", sizeof(GtkFileLauncherClass), alignof(GtkFileLauncherClass));
    printf("%s;%zu;%zu\n", "GtkFilter", sizeof(GtkFilter), alignof(GtkFilter));
    printf("%s;%zu;%zu\n", "GtkFilterChange", sizeof(GtkFilterChange), alignof(GtkFilterChange));
    printf("%s;%zu;%zu\n", "GtkFilterClass", sizeof(GtkFilterClass), alignof(GtkFilterClass));
    printf("%s;%zu;%zu\n", "GtkFilterListModelClass", sizeof(GtkFilterListModelClass), alignof(GtkFilterListModelClass));
    printf("%s;%zu;%zu\n", "GtkFilterMatch", sizeof(GtkFilterMatch), alignof(GtkFilterMatch));
    printf("%s;%zu;%zu\n", "GtkFixed", sizeof(GtkFixed), alignof(GtkFixed));
    printf("%s;%zu;%zu\n", "GtkFixedClass", sizeof(GtkFixedClass), alignof(GtkFixedClass));
    printf("%s;%zu;%zu\n", "GtkFixedLayoutChildClass", sizeof(GtkFixedLayoutChildClass), alignof(GtkFixedLayoutChildClass));
    printf("%s;%zu;%zu\n", "GtkFixedLayoutClass", sizeof(GtkFixedLayoutClass), alignof(GtkFixedLayoutClass));
    printf("%s;%zu;%zu\n", "GtkFlattenListModelClass", sizeof(GtkFlattenListModelClass), alignof(GtkFlattenListModelClass));
    printf("%s;%zu;%zu\n", "GtkFlowBoxChild", sizeof(GtkFlowBoxChild), alignof(GtkFlowBoxChild));
    printf("%s;%zu;%zu\n", "GtkFlowBoxChildClass", sizeof(GtkFlowBoxChildClass), alignof(GtkFlowBoxChildClass));
    printf("%s;%zu;%zu\n", "GtkFontChooserIface", sizeof(GtkFontChooserIface), alignof(GtkFontChooserIface));
    printf("%s;%zu;%zu\n", "GtkFontChooserLevel", sizeof(GtkFontChooserLevel), alignof(GtkFontChooserLevel));
    printf("%s;%zu;%zu\n", "GtkFontDialogButtonClass", sizeof(GtkFontDialogButtonClass), alignof(GtkFontDialogButtonClass));
    printf("%s;%zu;%zu\n", "GtkFontDialogClass", sizeof(GtkFontDialogClass), alignof(GtkFontDialogClass));
    printf("%s;%zu;%zu\n", "GtkFontLevel", sizeof(GtkFontLevel), alignof(GtkFontLevel));
    printf("%s;%zu;%zu\n", "GtkFrame", sizeof(GtkFrame), alignof(GtkFrame));
    printf("%s;%zu;%zu\n", "GtkFrameClass", sizeof(GtkFrameClass), alignof(GtkFrameClass));
    printf("%s;%zu;%zu\n", "GtkGLArea", sizeof(GtkGLArea), alignof(GtkGLArea));
    printf("%s;%zu;%zu\n", "GtkGLAreaClass", sizeof(GtkGLAreaClass), alignof(GtkGLAreaClass));
    printf("%s;%zu;%zu\n", "GtkGrid", sizeof(GtkGrid), alignof(GtkGrid));
    printf("%s;%zu;%zu\n", "GtkGridClass", sizeof(GtkGridClass), alignof(GtkGridClass));
    printf("%s;%zu;%zu\n", "GtkGridLayoutChildClass", sizeof(GtkGridLayoutChildClass), alignof(GtkGridLayoutChildClass));
    printf("%s;%zu;%zu\n", "GtkGridLayoutClass", sizeof(GtkGridLayoutClass), alignof(GtkGridLayoutClass));
    printf("%s;%zu;%zu\n", "GtkIMContext", sizeof(GtkIMContext), alignof(GtkIMContext));
    printf("%s;%zu;%zu\n", "GtkIMContextClass", sizeof(GtkIMContextClass), alignof(GtkIMContextClass));
    printf("%s;%zu;%zu\n", "GtkIMContextSimple", sizeof(GtkIMContextSimple), alignof(GtkIMContextSimple));
    printf("%s;%zu;%zu\n", "GtkIMContextSimpleClass", sizeof(GtkIMContextSimpleClass), alignof(GtkIMContextSimpleClass));
    printf("%s;%zu;%zu\n", "GtkIMMulticontext", sizeof(GtkIMMulticontext), alignof(GtkIMMulticontext));
    printf("%s;%zu;%zu\n", "GtkIMMulticontextClass", sizeof(GtkIMMulticontextClass), alignof(GtkIMMulticontextClass));
    printf("%s;%zu;%zu\n", "GtkIconLookupFlags", sizeof(GtkIconLookupFlags), alignof(GtkIconLookupFlags));
    printf("%s;%zu;%zu\n", "GtkIconSize", sizeof(GtkIconSize), alignof(GtkIconSize));
    printf("%s;%zu;%zu\n", "GtkIconThemeError", sizeof(GtkIconThemeError), alignof(GtkIconThemeError));
    printf("%s;%zu;%zu\n", "GtkIconViewDropPosition", sizeof(GtkIconViewDropPosition), alignof(GtkIconViewDropPosition));
    printf("%s;%zu;%zu\n", "GtkImageType", sizeof(GtkImageType), alignof(GtkImageType));
    printf("%s;%zu;%zu\n", "GtkInputHints", sizeof(GtkInputHints), alignof(GtkInputHints));
    printf("%s;%zu;%zu\n", "GtkInputPurpose", sizeof(GtkInputPurpose), alignof(GtkInputPurpose));
    printf("%s;%zu;%zu\n", "GtkInscriptionClass", sizeof(GtkInscriptionClass), alignof(GtkInscriptionClass));
    printf("%s;%zu;%zu\n", "GtkInscriptionOverflow", sizeof(GtkInscriptionOverflow), alignof(GtkInscriptionOverflow));
    printf("%s;%zu;%zu\n", "GtkJustification", sizeof(GtkJustification), alignof(GtkJustification));
    printf("%s;%zu;%zu\n", "GtkLayoutChild", sizeof(GtkLayoutChild), alignof(GtkLayoutChild));
    printf("%s;%zu;%zu\n", "GtkLayoutChildClass", sizeof(GtkLayoutChildClass), alignof(GtkLayoutChildClass));
    printf("%s;%zu;%zu\n", "GtkLayoutManager", sizeof(GtkLayoutManager), alignof(GtkLayoutManager));
    printf("%s;%zu;%zu\n", "GtkLayoutManagerClass", sizeof(GtkLayoutManagerClass), alignof(GtkLayoutManagerClass));
    printf("%s;%zu;%zu\n", "GtkLevelBarMode", sizeof(GtkLevelBarMode), alignof(GtkLevelBarMode));
    printf("%s;%zu;%zu\n", "GtkLicense", sizeof(GtkLicense), alignof(GtkLicense));
    printf("%s;%zu;%zu\n", "GtkListBoxRow", sizeof(GtkListBoxRow), alignof(GtkListBoxRow));
    printf("%s;%zu;%zu\n", "GtkListBoxRowClass", sizeof(GtkListBoxRowClass), alignof(GtkListBoxRowClass));
    printf("%s;%zu;%zu\n", "GtkListStore", sizeof(GtkListStore), alignof(GtkListStore));
    printf("%s;%zu;%zu\n", "GtkListStoreClass", sizeof(GtkListStoreClass), alignof(GtkListStoreClass));
    printf("%s;%zu;%zu\n", "GtkMapListModelClass", sizeof(GtkMapListModelClass), alignof(GtkMapListModelClass));
    printf("%s;%zu;%zu\n", "GtkMediaControlsClass", sizeof(GtkMediaControlsClass), alignof(GtkMediaControlsClass));
    printf("%s;%zu;%zu\n", "GtkMediaFile", sizeof(GtkMediaFile), alignof(GtkMediaFile));
    printf("%s;%zu;%zu\n", "GtkMediaFileClass", sizeof(GtkMediaFileClass), alignof(GtkMediaFileClass));
    printf("%s;%zu;%zu\n", "GtkMediaStream", sizeof(GtkMediaStream), alignof(GtkMediaStream));
    printf("%s;%zu;%zu\n", "GtkMediaStreamClass", sizeof(GtkMediaStreamClass), alignof(GtkMediaStreamClass));
    printf("%s;%zu;%zu\n", "GtkMessageDialog", sizeof(GtkMessageDialog), alignof(GtkMessageDialog));
    printf("%s;%zu;%zu\n", "GtkMessageType", sizeof(GtkMessageType), alignof(GtkMessageType));
    printf("%s;%zu;%zu\n", "GtkMountOperation", sizeof(GtkMountOperation), alignof(GtkMountOperation));
    printf("%s;%zu;%zu\n", "GtkMountOperationClass", sizeof(GtkMountOperationClass), alignof(GtkMountOperationClass));
    printf("%s;%zu;%zu\n", "GtkMovementStep", sizeof(GtkMovementStep), alignof(GtkMovementStep));
    printf("%s;%zu;%zu\n", "GtkMultiSelectionClass", sizeof(GtkMultiSelectionClass), alignof(GtkMultiSelectionClass));
    printf("%s;%zu;%zu\n", "GtkMultiSorterClass", sizeof(GtkMultiSorterClass), alignof(GtkMultiSorterClass));
    printf("%s;%zu;%zu\n", "GtkNativeDialog", sizeof(GtkNativeDialog), alignof(GtkNativeDialog));
    printf("%s;%zu;%zu\n", "GtkNativeDialogClass", sizeof(GtkNativeDialogClass), alignof(GtkNativeDialogClass));
    printf("%s;%zu;%zu\n", "GtkNaturalWrapMode", sizeof(GtkNaturalWrapMode), alignof(GtkNaturalWrapMode));
    printf("%s;%zu;%zu\n", "GtkNoSelectionClass", sizeof(GtkNoSelectionClass), alignof(GtkNoSelectionClass));
    printf("%s;%zu;%zu\n", "GtkNotebookTab", sizeof(GtkNotebookTab), alignof(GtkNotebookTab));
    printf("%s;%zu;%zu\n", "GtkNumberUpLayout", sizeof(GtkNumberUpLayout), alignof(GtkNumberUpLayout));
    printf("%s;%zu;%zu\n", "GtkNumericSorterClass", sizeof(GtkNumericSorterClass), alignof(GtkNumericSorterClass));
    printf("%s;%zu;%zu\n", "GtkOrdering", sizeof(GtkOrdering), alignof(GtkOrdering));
    printf("%s;%zu;%zu\n", "GtkOrientableIface", sizeof(GtkOrientableIface), alignof(GtkOrientableIface));
    printf("%s;%zu;%zu\n", "GtkOrientation", sizeof(GtkOrientation), alignof(GtkOrientation));
    printf("%s;%zu;%zu\n", "GtkOverflow", sizeof(GtkOverflow), alignof(GtkOverflow));
    printf("%s;%zu;%zu\n", "GtkOverlayLayoutChildClass", sizeof(GtkOverlayLayoutChildClass), alignof(GtkOverlayLayoutChildClass));
    printf("%s;%zu;%zu\n", "GtkOverlayLayoutClass", sizeof(GtkOverlayLayoutClass), alignof(GtkOverlayLayoutClass));
    printf("%s;%zu;%zu\n", "GtkPackType", sizeof(GtkPackType), alignof(GtkPackType));
    printf("%s;%zu;%zu\n", "GtkPadActionEntry", sizeof(GtkPadActionEntry), alignof(GtkPadActionEntry));
    printf("%s;%zu;%zu\n", "GtkPadActionType", sizeof(GtkPadActionType), alignof(GtkPadActionType));
    printf("%s;%zu;%zu\n", "GtkPageOrientation", sizeof(GtkPageOrientation), alignof(GtkPageOrientation));
    printf("%s;%zu;%zu\n", "GtkPageRange", sizeof(GtkPageRange), alignof(GtkPageRange));
    printf("%s;%zu;%zu\n", "GtkPageSet", sizeof(GtkPageSet), alignof(GtkPageSet));
    printf("%s;%zu;%zu\n", "GtkPanDirection", sizeof(GtkPanDirection), alignof(GtkPanDirection));
    printf("%s;%zu;%zu\n", "GtkParamSpecExpression", sizeof(GtkParamSpecExpression), alignof(GtkParamSpecExpression));
    printf("%s;%zu;%zu\n", "GtkPasswordEntryBufferClass", sizeof(GtkPasswordEntryBufferClass), alignof(GtkPasswordEntryBufferClass));
    printf("%s;%zu;%zu\n", "GtkPickFlags", sizeof(GtkPickFlags), alignof(GtkPickFlags));
    printf("%s;%zu;%zu\n", "GtkPictureClass", sizeof(GtkPictureClass), alignof(GtkPictureClass));
    printf("%s;%zu;%zu\n", "GtkPolicyType", sizeof(GtkPolicyType), alignof(GtkPolicyType));
    printf("%s;%zu;%zu\n", "GtkPopover", sizeof(GtkPopover), alignof(GtkPopover));
    printf("%s;%zu;%zu\n", "GtkPopoverClass", sizeof(GtkPopoverClass), alignof(GtkPopoverClass));
    printf("%s;%zu;%zu\n", "GtkPopoverMenuFlags", sizeof(GtkPopoverMenuFlags), alignof(GtkPopoverMenuFlags));
    printf("%s;%zu;%zu\n", "GtkPositionType", sizeof(GtkPositionType), alignof(GtkPositionType));
    printf("%s;%zu;%zu\n", "GtkPrintCapabilities", sizeof(GtkPrintCapabilities), alignof(GtkPrintCapabilities));
    printf("%s;%zu;%zu\n", "GtkPrintDuplex", sizeof(GtkPrintDuplex), alignof(GtkPrintDuplex));
    printf("%s;%zu;%zu\n", "GtkPrintError", sizeof(GtkPrintError), alignof(GtkPrintError));
    printf("%s;%zu;%zu\n", "GtkPrintOperation", sizeof(GtkPrintOperation), alignof(GtkPrintOperation));
    printf("%s;%zu;%zu\n", "GtkPrintOperationAction", sizeof(GtkPrintOperationAction), alignof(GtkPrintOperationAction));
    printf("%s;%zu;%zu\n", "GtkPrintOperationClass", sizeof(GtkPrintOperationClass), alignof(GtkPrintOperationClass));
    printf("%s;%zu;%zu\n", "GtkPrintOperationPreviewIface", sizeof(GtkPrintOperationPreviewIface), alignof(GtkPrintOperationPreviewIface));
    printf("%s;%zu;%zu\n", "GtkPrintOperationResult", sizeof(GtkPrintOperationResult), alignof(GtkPrintOperationResult));
    printf("%s;%zu;%zu\n", "GtkPrintPages", sizeof(GtkPrintPages), alignof(GtkPrintPages));
    printf("%s;%zu;%zu\n", "GtkPrintQuality", sizeof(GtkPrintQuality), alignof(GtkPrintQuality));
    printf("%s;%zu;%zu\n", "GtkPrintStatus", sizeof(GtkPrintStatus), alignof(GtkPrintStatus));
    printf("%s;%zu;%zu\n", "GtkPropagationLimit", sizeof(GtkPropagationLimit), alignof(GtkPropagationLimit));
    printf("%s;%zu;%zu\n", "GtkPropagationPhase", sizeof(GtkPropagationPhase), alignof(GtkPropagationPhase));
    printf("%s;%zu;%zu\n", "GtkRange", sizeof(GtkRange), alignof(GtkRange));
    printf("%s;%zu;%zu\n", "GtkRangeClass", sizeof(GtkRangeClass), alignof(GtkRangeClass));
    printf("%s;%zu;%zu\n", "GtkRecentData", sizeof(GtkRecentData), alignof(GtkRecentData));
    printf("%s;%zu;%zu\n", "GtkRecentManager", sizeof(GtkRecentManager), alignof(GtkRecentManager));
    printf("%s;%zu;%zu\n", "GtkRecentManagerClass", sizeof(GtkRecentManagerClass), alignof(GtkRecentManagerClass));
    printf("%s;%zu;%zu\n", "GtkRecentManagerError", sizeof(GtkRecentManagerError), alignof(GtkRecentManagerError));
    printf("%s;%zu;%zu\n", "GtkRequestedSize", sizeof(GtkRequestedSize), alignof(GtkRequestedSize));
    printf("%s;%zu;%zu\n", "GtkRequisition", sizeof(GtkRequisition), alignof(GtkRequisition));
    printf("%s;%zu;%zu\n", "GtkResponseType", sizeof(GtkResponseType), alignof(GtkResponseType));
    printf("%s;%zu;%zu\n", "GtkRevealerTransitionType", sizeof(GtkRevealerTransitionType), alignof(GtkRevealerTransitionType));
    printf("%s;%zu;%zu\n", "GtkScale", sizeof(GtkScale), alignof(GtkScale));
    printf("%s;%zu;%zu\n", "GtkScaleButton", sizeof(GtkScaleButton), alignof(GtkScaleButton));
    printf("%s;%zu;%zu\n", "GtkScaleButtonClass", sizeof(GtkScaleButtonClass), alignof(GtkScaleButtonClass));
    printf("%s;%zu;%zu\n", "GtkScaleClass", sizeof(GtkScaleClass), alignof(GtkScaleClass));
    printf("%s;%zu;%zu\n", "GtkScrollStep", sizeof(GtkScrollStep), alignof(GtkScrollStep));
    printf("%s;%zu;%zu\n", "GtkScrollType", sizeof(GtkScrollType), alignof(GtkScrollType));
    printf("%s;%zu;%zu\n", "GtkScrollableInterface", sizeof(GtkScrollableInterface), alignof(GtkScrollableInterface));
    printf("%s;%zu;%zu\n", "GtkScrollablePolicy", sizeof(GtkScrollablePolicy), alignof(GtkScrollablePolicy));
    printf("%s;%zu;%zu\n", "GtkSelectionFilterModelClass", sizeof(GtkSelectionFilterModelClass), alignof(GtkSelectionFilterModelClass));
    printf("%s;%zu;%zu\n", "GtkSelectionMode", sizeof(GtkSelectionMode), alignof(GtkSelectionMode));
    printf("%s;%zu;%zu\n", "GtkSelectionModelInterface", sizeof(GtkSelectionModelInterface), alignof(GtkSelectionModelInterface));
    printf("%s;%zu;%zu\n", "GtkSensitivityType", sizeof(GtkSensitivityType), alignof(GtkSensitivityType));
    printf("%s;%zu;%zu\n", "GtkShortcutActionFlags", sizeof(GtkShortcutActionFlags), alignof(GtkShortcutActionFlags));
    printf("%s;%zu;%zu\n", "GtkShortcutClass", sizeof(GtkShortcutClass), alignof(GtkShortcutClass));
    printf("%s;%zu;%zu\n", "GtkShortcutManagerInterface", sizeof(GtkShortcutManagerInterface), alignof(GtkShortcutManagerInterface));
    printf("%s;%zu;%zu\n", "GtkShortcutScope", sizeof(GtkShortcutScope), alignof(GtkShortcutScope));
    printf("%s;%zu;%zu\n", "GtkShortcutType", sizeof(GtkShortcutType), alignof(GtkShortcutType));
    printf("%s;%zu;%zu\n", "GtkSingleSelectionClass", sizeof(GtkSingleSelectionClass), alignof(GtkSingleSelectionClass));
    printf("%s;%zu;%zu\n", "GtkSizeGroup", sizeof(GtkSizeGroup), alignof(GtkSizeGroup));
    printf("%s;%zu;%zu\n", "GtkSizeGroupMode", sizeof(GtkSizeGroupMode), alignof(GtkSizeGroupMode));
    printf("%s;%zu;%zu\n", "GtkSizeRequestMode", sizeof(GtkSizeRequestMode), alignof(GtkSizeRequestMode));
    printf("%s;%zu;%zu\n", "GtkSliceListModelClass", sizeof(GtkSliceListModelClass), alignof(GtkSliceListModelClass));
    printf("%s;%zu;%zu\n", "GtkSortListModelClass", sizeof(GtkSortListModelClass), alignof(GtkSortListModelClass));
    printf("%s;%zu;%zu\n", "GtkSortType", sizeof(GtkSortType), alignof(GtkSortType));
    printf("%s;%zu;%zu\n", "GtkSorter", sizeof(GtkSorter), alignof(GtkSorter));
    printf("%s;%zu;%zu\n", "GtkSorterChange", sizeof(GtkSorterChange), alignof(GtkSorterChange));
    printf("%s;%zu;%zu\n", "GtkSorterClass", sizeof(GtkSorterClass), alignof(GtkSorterClass));
    printf("%s;%zu;%zu\n", "GtkSorterOrder", sizeof(GtkSorterOrder), alignof(GtkSorterOrder));
    printf("%s;%zu;%zu\n", "GtkSpinButtonUpdatePolicy", sizeof(GtkSpinButtonUpdatePolicy), alignof(GtkSpinButtonUpdatePolicy));
    printf("%s;%zu;%zu\n", "GtkSpinType", sizeof(GtkSpinType), alignof(GtkSpinType));
    printf("%s;%zu;%zu\n", "GtkStackTransitionType", sizeof(GtkStackTransitionType), alignof(GtkStackTransitionType));
    printf("%s;%zu;%zu\n", "GtkStateFlags", sizeof(GtkStateFlags), alignof(GtkStateFlags));
    printf("%s;%zu;%zu\n", "GtkStringFilterClass", sizeof(GtkStringFilterClass), alignof(GtkStringFilterClass));
    printf("%s;%zu;%zu\n", "GtkStringFilterMatchMode", sizeof(GtkStringFilterMatchMode), alignof(GtkStringFilterMatchMode));
    printf("%s;%zu;%zu\n", "GtkStringListClass", sizeof(GtkStringListClass), alignof(GtkStringListClass));
    printf("%s;%zu;%zu\n", "GtkStringObjectClass", sizeof(GtkStringObjectClass), alignof(GtkStringObjectClass));
    printf("%s;%zu;%zu\n", "GtkStringSorterClass", sizeof(GtkStringSorterClass), alignof(GtkStringSorterClass));
    printf("%s;%zu;%zu\n", "GtkStyleContext", sizeof(GtkStyleContext), alignof(GtkStyleContext));
    printf("%s;%zu;%zu\n", "GtkStyleContextClass", sizeof(GtkStyleContextClass), alignof(GtkStyleContextClass));
    printf("%s;%zu;%zu\n", "GtkStyleContextPrintFlags", sizeof(GtkStyleContextPrintFlags), alignof(GtkStyleContextPrintFlags));
    printf("%s;%zu;%zu\n", "GtkSymbolicColor", sizeof(GtkSymbolicColor), alignof(GtkSymbolicColor));
    printf("%s;%zu;%zu\n", "GtkSymbolicPaintableInterface", sizeof(GtkSymbolicPaintableInterface), alignof(GtkSymbolicPaintableInterface));
    printf("%s;%zu;%zu\n", "GtkSystemSetting", sizeof(GtkSystemSetting), alignof(GtkSystemSetting));
    printf("%s;%zu;%zu\n", "GtkText", sizeof(GtkText), alignof(GtkText));
    printf("%s;%zu;%zu\n", "GtkTextBuffer", sizeof(GtkTextBuffer), alignof(GtkTextBuffer));
    printf("%s;%zu;%zu\n", "GtkTextBufferClass", sizeof(GtkTextBufferClass), alignof(GtkTextBufferClass));
    printf("%s;%zu;%zu\n", "GtkTextChildAnchor", sizeof(GtkTextChildAnchor), alignof(GtkTextChildAnchor));
    printf("%s;%zu;%zu\n", "GtkTextChildAnchorClass", sizeof(GtkTextChildAnchorClass), alignof(GtkTextChildAnchorClass));
    printf("%s;%zu;%zu\n", "GtkTextDirection", sizeof(GtkTextDirection), alignof(GtkTextDirection));
    printf("%s;%zu;%zu\n", "GtkTextExtendSelection", sizeof(GtkTextExtendSelection), alignof(GtkTextExtendSelection));
    printf("%s;%zu;%zu\n", "GtkTextIter", sizeof(GtkTextIter), alignof(GtkTextIter));
    printf("%s;%zu;%zu\n", "GtkTextMark", sizeof(GtkTextMark), alignof(GtkTextMark));
    printf("%s;%zu;%zu\n", "GtkTextMarkClass", sizeof(GtkTextMarkClass), alignof(GtkTextMarkClass));
    printf("%s;%zu;%zu\n", "GtkTextSearchFlags", sizeof(GtkTextSearchFlags), alignof(GtkTextSearchFlags));
    printf("%s;%zu;%zu\n", "GtkTextTag", sizeof(GtkTextTag), alignof(GtkTextTag));
    printf("%s;%zu;%zu\n", "GtkTextTagClass", sizeof(GtkTextTagClass), alignof(GtkTextTagClass));
    printf("%s;%zu;%zu\n", "GtkTextView", sizeof(GtkTextView), alignof(GtkTextView));
    printf("%s;%zu;%zu\n", "GtkTextViewClass", sizeof(GtkTextViewClass), alignof(GtkTextViewClass));
    printf("%s;%zu;%zu\n", "GtkTextViewLayer", sizeof(GtkTextViewLayer), alignof(GtkTextViewLayer));
    printf("%s;%zu;%zu\n", "GtkTextWindowType", sizeof(GtkTextWindowType), alignof(GtkTextWindowType));
    printf("%s;%zu;%zu\n", "GtkToggleButton", sizeof(GtkToggleButton), alignof(GtkToggleButton));
    printf("%s;%zu;%zu\n", "GtkToggleButtonClass", sizeof(GtkToggleButtonClass), alignof(GtkToggleButtonClass));
    printf("%s;%zu;%zu\n", "GtkTreeDragDestIface", sizeof(GtkTreeDragDestIface), alignof(GtkTreeDragDestIface));
    printf("%s;%zu;%zu\n", "GtkTreeDragSourceIface", sizeof(GtkTreeDragSourceIface), alignof(GtkTreeDragSourceIface));
    printf("%s;%zu;%zu\n", "GtkTreeExpanderClass", sizeof(GtkTreeExpanderClass), alignof(GtkTreeExpanderClass));
    printf("%s;%zu;%zu\n", "GtkTreeIter", sizeof(GtkTreeIter), alignof(GtkTreeIter));
    printf("%s;%zu;%zu\n", "GtkTreeListModelClass", sizeof(GtkTreeListModelClass), alignof(GtkTreeListModelClass));
    printf("%s;%zu;%zu\n", "GtkTreeListRowClass", sizeof(GtkTreeListRowClass), alignof(GtkTreeListRowClass));
    printf("%s;%zu;%zu\n", "GtkTreeListRowSorterClass", sizeof(GtkTreeListRowSorterClass), alignof(GtkTreeListRowSorterClass));
    printf("%s;%zu;%zu\n", "GtkTreeModelFilter", sizeof(GtkTreeModelFilter), alignof(GtkTreeModelFilter));
    printf("%s;%zu;%zu\n", "GtkTreeModelFilterClass", sizeof(GtkTreeModelFilterClass), alignof(GtkTreeModelFilterClass));
    printf("%s;%zu;%zu\n", "GtkTreeModelFlags", sizeof(GtkTreeModelFlags), alignof(GtkTreeModelFlags));
    printf("%s;%zu;%zu\n", "GtkTreeModelIface", sizeof(GtkTreeModelIface), alignof(GtkTreeModelIface));
    printf("%s;%zu;%zu\n", "GtkTreeModelSort", sizeof(GtkTreeModelSort), alignof(GtkTreeModelSort));
    printf("%s;%zu;%zu\n", "GtkTreeModelSortClass", sizeof(GtkTreeModelSortClass), alignof(GtkTreeModelSortClass));
    printf("%s;%zu;%zu\n", "GtkTreeSortableIface", sizeof(GtkTreeSortableIface), alignof(GtkTreeSortableIface));
    printf("%s;%zu;%zu\n", "GtkTreeStore", sizeof(GtkTreeStore), alignof(GtkTreeStore));
    printf("%s;%zu;%zu\n", "GtkTreeStoreClass", sizeof(GtkTreeStoreClass), alignof(GtkTreeStoreClass));
    printf("%s;%zu;%zu\n", "GtkTreeView", sizeof(GtkTreeView), alignof(GtkTreeView));
    printf("%s;%zu;%zu\n", "GtkTreeViewClass", sizeof(GtkTreeViewClass), alignof(GtkTreeViewClass));
    printf("%s;%zu;%zu\n", "GtkTreeViewColumnSizing", sizeof(GtkTreeViewColumnSizing), alignof(GtkTreeViewColumnSizing));
    printf("%s;%zu;%zu\n", "GtkTreeViewDropPosition", sizeof(GtkTreeViewDropPosition), alignof(GtkTreeViewDropPosition));
    printf("%s;%zu;%zu\n", "GtkTreeViewGridLines", sizeof(GtkTreeViewGridLines), alignof(GtkTreeViewGridLines));
    printf("%s;%zu;%zu\n", "GtkUnit", sizeof(GtkUnit), alignof(GtkUnit));
    printf("%s;%zu;%zu\n", "GtkUriLauncherClass", sizeof(GtkUriLauncherClass), alignof(GtkUriLauncherClass));
    printf("%s;%zu;%zu\n", "GtkVideoClass", sizeof(GtkVideoClass), alignof(GtkVideoClass));
    printf("%s;%zu;%zu\n", "GtkVolumeButton", sizeof(GtkVolumeButton), alignof(GtkVolumeButton));
    printf("%s;%zu;%zu\n", "GtkWidget", sizeof(GtkWidget), alignof(GtkWidget));
    printf("%s;%zu;%zu\n", "GtkWidgetClass", sizeof(GtkWidgetClass), alignof(GtkWidgetClass));
    printf("%s;%zu;%zu\n", "GtkWidgetPaintableClass", sizeof(GtkWidgetPaintableClass), alignof(GtkWidgetPaintableClass));
    printf("%s;%zu;%zu\n", "GtkWindow", sizeof(GtkWindow), alignof(GtkWindow));
    printf("%s;%zu;%zu\n", "GtkWindowClass", sizeof(GtkWindowClass), alignof(GtkWindowClass));
    printf("%s;%zu;%zu\n", "GtkWindowControlsClass", sizeof(GtkWindowControlsClass), alignof(GtkWindowControlsClass));
    printf("%s;%zu;%zu\n", "GtkWindowGroup", sizeof(GtkWindowGroup), alignof(GtkWindowGroup));
    printf("%s;%zu;%zu\n", "GtkWindowGroupClass", sizeof(GtkWindowGroupClass), alignof(GtkWindowGroupClass));
    printf("%s;%zu;%zu\n", "GtkWindowHandleClass", sizeof(GtkWindowHandleClass), alignof(GtkWindowHandleClass));
    printf("%s;%zu;%zu\n", "GtkWrapMode", sizeof(GtkWrapMode), alignof(GtkWrapMode));
    return 0;
}
