// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

#include "manual.h"
#include <stdalign.h>
#include <stdio.h>

int main() {
    printf("%s;%zu;%zu\n", "GskBlendMode", sizeof(GskBlendMode), alignof(GskBlendMode));
    printf("%s;%zu;%zu\n", "GskColorStop", sizeof(GskColorStop), alignof(GskColorStop));
    printf("%s;%zu;%zu\n", "GskCorner", sizeof(GskCorner), alignof(GskCorner));
    printf("%s;%zu;%zu\n", "GskGLShaderClass", sizeof(GskGLShaderClass), alignof(GskGLShaderClass));
    printf("%s;%zu;%zu\n", "GskGLUniformType", sizeof(GskGLUniformType), alignof(GskGLUniformType));
    printf("%s;%zu;%zu\n", "GskMaskMode", sizeof(GskMaskMode), alignof(GskMaskMode));
    printf("%s;%zu;%zu\n", "GskParseLocation", sizeof(GskParseLocation), alignof(GskParseLocation));
    printf("%s;%zu;%zu\n", "GskRenderNodeType", sizeof(GskRenderNodeType), alignof(GskRenderNodeType));
    printf("%s;%zu;%zu\n", "GskRoundedRect", sizeof(GskRoundedRect), alignof(GskRoundedRect));
    printf("%s;%zu;%zu\n", "GskScalingFilter", sizeof(GskScalingFilter), alignof(GskScalingFilter));
    printf("%s;%zu;%zu\n", "GskSerializationError", sizeof(GskSerializationError), alignof(GskSerializationError));
    printf("%s;%zu;%zu\n", "GskShadow", sizeof(GskShadow), alignof(GskShadow));
    printf("%s;%zu;%zu\n", "GskTransformCategory", sizeof(GskTransformCategory), alignof(GskTransformCategory));
    return 0;
}
